# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv

import base64
from tempfile import TemporaryFile
import csv

from tools.translate import _

class import_bank_statement(osv.osv):
    _name = 'account.import_bank_statement'
    _description = 'Import Bank Statement'
    _inherit = ['mail.thread']

    def default_state(self, cr, uid, context={}):
        return 'draft'

    def default_company_id(self, cr, uid, context={}):
        obj_user = self.pool.get('res.users')

        user = obj_user.browse(cr, uid, [uid])[0]

        return user.company_id.id

    _columns =  {
                'name' : fields.char(string='Description', size=100, required=True, readonly=True, states={'draft':[('readonly',False)]}),
                'company_id' : fields.many2one(string='Company', obj='res.company', required=True, readonly=True, states={'draft':[('readonly',False)]}),
                'fiscalyear_id' : fields.many2one(string='Fiscal Year', obj='account.fiscalyear', required=True, readonly=True, states={'draft':[('readonly',False)]}),
                'period_id' : fields.many2one(string='Period', obj='account.period', required=True, readonly=True, states={'draft':[('readonly',False)]}),
                'file_name' : fields.binary(string='Bank Statement File', required=True, readonly=True, states={'draft':[('readonly',False)]}),
                'journal_id' : fields.many2one(string='Journal', obj='account.journal', required=True, readonly=True, states={'draft':[('readonly',False)]}),
                'temporary_account_id' : fields.many2one(string='Temporary Account', obj='account.account', required=True, readonly=True, states={'draft':[('readonly',False)]}),
                'bank_statement_type_id' : fields.many2one(string='Statement Type', obj='account.bank_statement_type', required=True, readonly=True, states={'draft':[('readonly',False)]}),
                'bank_statement_type' : fields.related('bank_statement_type_id','name', string='Statement Type', type='char', relation='account.bank_statement_type', store=True, readonly=True),
                'import_bank_statement_ids' : fields.one2many(string='Detail', obj='account.import_bank_statement_detail', fields_id='import_bank_statement_id'),
                'error_message' : fields.text(string='Error Message', readonly=True),
                'state' : fields.selection(string='State', selection=[('draft','Draft'),('imported','Imported'),('open','On Progress'),('done','Done'),('cancel','Cancelled')], readonly=True, required=True),
                }


    _defaults = {
                'state' : default_state,
                'company_id' : default_company_id,
                }

    def unlink(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('Warning!'),_('You can not delete data, please cancel instead'))
        return False

    def copy(self, cr, uid, id, defaults, context=None):
        raise osv.except_osv(_('Warning!'),_('You can not copy data'))
        return False


    def button_import(self, cr, uid, ids, context={}):
        for id in ids:

            csv_file = False

            if not self._clear_detail(cr, uid, id):
                return False

            csv_file = self._open_file(cr, uid, id)
            if not csv_file:
                return False

            csv_file = self._cleanup_csv(cr, uid, id, csv_file)
            if not csv_file:
                return False

            csv_file = self._read_csv_row(cr, uid, id, csv_file)
            if not csv_file:
                return False

            if not self._import_result_list(cr, uid, id, csv_file):
                return False

        return True

    def button_posted(self, cr, uid, ids, context={}):
        for id in ids:
            if not self._create_bank_voucher(cr, uid, id):
                return False

        return True

    def button_confirm(self, cr, uid, ids, context={}):
        for id in ids:
            self.write(cr, uid, [id], {'state' : 'imported'})

        return True

    def button_finished_assign_voucher(self, cr, uid, ids, context={}):
        for id in ids:
            if not self._check_finished_assign(cr, uid, id):
                return False

            self.write(cr, uid, [id], {'state' : 'open'})

        return True

    def button_done(self, cr, uid, ids, context={}):
        for id in ids:
            if not self._check_voucher_posted(cr, uid, id):
                return False

            self.write(cr, uid, [id], {'state' : 'done'})

        return True

    def button_cancel(self, cr, uid, ids, context={}):
        for id in ids:
            if not self._check_voucher_cancel(cr, uid, id):
                return False

            self.write(cr, uid, [id], {'state' : 'cancel'})

        return True

    def button_set_to_draft(self, cr, uid, ids, context={}):
        for id in ids:
            self.write(cr, uid, [id], {'state' : 'draft'})

        return True

    def _import_csv(self, cr, uid, id):
        #TODO
        return True

    def _create_bank_voucher(self, cr, uid, id):
        obj_detail = self.pool.get('account.import_bank_statement_detail')

        import_bank_statement = self.browse(cr, uid, [id])[0]

        if import_bank_statement.import_bank_statement_ids:
            for detail in import_bank_statement.import_bank_statement_ids:
                if not obj_detail._create_account_voucher(cr, uid, detail.id):
                    return False

        return True

    def _open_file(self, cr, uid, id):
        obj_import_bank_statement = self.pool.get('account.import_bank_statement')

        import_bank_statement = obj_import_bank_statement.browse(cr, uid, [id])[0]

        try:
            fileobj = TemporaryFile('w+')
            fileobj.write(base64.decodestring(import_bank_statement.file_name))
        except Exception, e:
            raise osv.except_osv(_('Open file error'),_('%s') % e)

        return fileobj

    def _cleanup_csv(self, cr, uid, id, fileobj):
        # obj_import_bank_statement = self.pool.get('account.import_bank_statement')

        import_bank_statement = self.browse(cr, uid, [id])[0]

        if import_bank_statement.bank_statement_type_id.cleanup_method == 'method':
            return self._cleanup_csv_method(cr, uid, id, fileobj)
        elif import_bank_statement.bank_statement_type_id.cleanup_method == 'line':
            # raise osv.except_osv(_('Warning!'),_('Method is not supported yet'))
            return self._cleanup_csv_line(cr, uid, id, fileobj)
            # return False
        elif import_bank_statement.bank_statement_type_id.cleanup_method == 'custom':
            raise osv.except_osv(_('Warning!'),_('Custom method for CSV cleanup is not supported yet'))
            return False

    def _cleanup_csv_line(self, cr, uid, id, fileobj):
        obj_bank_statement_type = self.pool.get('account.bank_statement_type')

        import_bank_statement = self.browse(cr, uid, [id])[0]

        if not obj_bank_statement_type._check_line_value(cr, uid, import_bank_statement.bank_statement_type_id.id):
            return False

        index  = 0
        output = []


        fileobj.seek(0)
        for line in fileobj:
            index += 1
            if index > import_bank_statement.bank_statement_type_id.line_ignore:
                output.append(line)

        fileobj.close()
        fileobj = TemporaryFile('w+')
        fileobj.writelines(output)

        return fileobj


    def _cleanup_csv_method(self, cr, uid, id, fileobj):
        obj_import_bank_statement = self.pool.get('account.import_bank_statement')

        import_bank_statement = obj_import_bank_statement.browse(cr, uid, [id])[0]

        try:
            f = getattr(obj_import_bank_statement, import_bank_statement.bank_statement_type_id.csv_cleanup_function, None)
        except Exception, e:
            raise osv.except_osv(_('CSV cleanup method error'),_('%s') % e)
        else:
            result = f(cr, uid, id, fileobj)

            return result

    def _read_csv_row(self, cr, uid, id, fileobj):
        import_bank_statement = self.browse(cr, uid, [id])[0]

        if import_bank_statement.bank_statement_type_id.row_reading_method == 'method':
            return self._read_csv_row_method(cr, uid, id, fileobj)
        elif import_bank_statement.bank_statement_type_id.row_reading_method == 'index':
            return self._read_csv_row_index(cr, uid, id, fileobj)
        elif import_bank_statement.bank_statement_type_id.row_reading_method == 'custom':
            raise osv.except_osv(_('Warning!'),_('Custom method for row reading is not supported yet'))
            return False

    def _read_csv_row_index(self, cr, uid, id, fileobj):
        import_bank_statement = self.browse(cr, uid, [id])[0]
        bs_type = import_bank_statement.bank_statement_type_id

        list_result = []
        fileobj.seek(0)
        csvfile = csv.reader(fileobj,delimiter=';') #TODO: flexible delimiter

        name_index = bs_type.name_index - 1
        date_index = bs_type.date_index - 1

        if bs_type.amount_reading_method == 'dual':
            pass
        elif bs_type.amount_reading_method == 'single1':
            transaction_type_index = bs_type.transaction_type_index - 1

        if bs_type.amount_reading_method == 'dual':
            pass
        elif bs_type.amount_reading_method == 'single1' or bs_type.amount_reading_method == 'single2':
            amount_index = bs_type.amount_index - 1

        for baris in csvfile:
            if bs_type.amount_reading_method == 'single2':
                if float(baris[amount_index]) > 0.0:
                    transaction_type = 'dr'
                elif float(baris[amount_index]) < 0.0:
                    transaction_type = 'cr'
            elif bs_type.amount_reading_method == 'single1':
                transaction_type = baris[transaction_type_index]
            elif bs_type.amount_reading_method == 'dual':
                if float(baris[bs_type.dr_index - 1]) > 0.0:
                    transaction_type = 'dr'
                elif float(baris[bs_type.cr_index - 1]) > 0.0:
                    transaction_type = 'cr'


            if bs_type.amount_reading_method == 'single1' or bs_type.amount_reading_method == 'single2':
                amount = abs(float(baris[amount_index]))
            elif bs_type.amount_reading_method == 'dual':
                if float(baris[bs_type.dr_index - 1]) > 0.0:
                    amount = float(baris[bs_type.dr_index - 1])
                elif float(baris[bs_type.cr_index - 1]) > 0.0:
                    amount = float(baris[bs_type.cr_index - 1])

            res =   {
                    'name' : baris[name_index],
                    'date' : baris[date_index], #TODO: flexible date format
                    'transaction_type' : transaction_type,
                    'amount' : amount,
                    }
            list_result.append(res)

        return list_result

    def _read_csv_row_method(self, cr, uid, id, fileobj):
        obj_import_bank_statement = self.pool.get('account.import_bank_statement')

        import_bank_statement = obj_import_bank_statement.browse(cr, uid, [id])[0]

        try:
            f = getattr(obj_import_bank_statement, import_bank_statement.bank_statement_type_id.csv_read_row_function, None)
            result = f(cr, uid, id, fileobj)
        except Exception, e:
            raise osv.except_osv(_('CSV row reading error'),_('%s') % e)
        else:
            return result

    def _generic_cleanup_function(self, cr, uid, id, fileobj):
        try:
            result = fileobj
        except Exception, e:
            error_message = """
                            Error message : \n
                            %s
                            """ % (e)
            raise osv.except_osv(_('CSV row reading error'), error_message)
        else:
            return result

    def _generic_read_row_function(self, cr, uid, id, fileobj):
        list_result = []
        fileobj.seek(1)
        csvfile = csv.reader(fileobj,delimiter=';')

        for baris in csvfile:
            res =   {
                    'name' : baris[1],
                    'date' : baris[0],
                    'transaction_type' : baris[2],
                    'amount' : baris[3],
                    }
            list_result.append(res)

        list_result.pop(0)
        return list_result

    def _import_result_list(self, cr, uid, id, list_result):
        obj_detail = self.pool.get('account.import_bank_statement_detail')

        for import_row in list_result:
            import_row['import_bank_statement_id'] = id
            obj_detail.create(cr, uid, import_row)

        return True

    def _check_finished_assign(self, cr, uid, id):
        import_bank_statement = self.browse(cr, uid, [id])[0]

        for detail in import_bank_statement.import_bank_statement_ids:
            if not detail.account_voucher_id:
                raise osv.except_osv(_('Warning!'),_('Not all line have a voucher'))
                return False

        return True

    def _check_voucher_posted(self, cr, uid, id):
        import_bank_statement = self.browse(cr, uid, [id])[0]

        for detail in import_bank_statement.import_bank_statement_ids:
            if detail.account_voucher_id.state != 'posted':
                raise osv.except_osv(_('Warning!'),_('No all voucher is posted'))
                return False

        return True

    def _check_voucher_cancel(self, cr, uid, id):
        import_bank_statement = self.browse(cr, uid, [id])[0]

        for detail in import_bank_statement.import_bank_statement_ids:
            if detail.account_voucher_id and detail.account_voucher_id.state != 'cancel':
                raise osv.except_osv(_('Warning!'),_('Cancel or unassign voucher first'))
                return False

        return True

    def _clear_detail(self, cr, uid, id):
        obj_detail = self.pool.get('account.import_bank_statement_detail')

        criteria = [('import_bank_statement_id','=',id)]

        detail_ids = obj_detail.search(cr, uid, criteria)
        obj_detail.unlink(cr, uid, detail_ids)

        return True

import_bank_statement()

class import_bank_statement_detail(osv.osv):
    _name = 'account.import_bank_statement_detail'
    _description = 'Import Bank Statement Detail'

    def function_state(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        for detail in self.browse(cr, uid, ids):
            res[detail.id] = 'unassigned'

            if detail.account_voucher_id:
                res[detail.id] = detail.account_voucher_id.state

        return res

    def _get_import_bank_statement(self, cr, uid, ids, context=None):
        obj_import_bs = self.pool.get('account.import_bank_statement')
        result = []

        for import_bs in obj_import_bs.browse(cr, uid, ids):
            if import_bs.import_bank_statement_ids:
                for detail in import_bs.import_bank_statement_ids:
                    result.append(detail.import_bank_statement_id.id)

        return result

    def _get_account_voucher(self, cr, uid, ids, context=None):
        result = []
        obj_voucher = self.pool.get('account.voucher')
        obj_detail = self.pool.get('account.import_bank_statement_detail')

        for voucher in obj_voucher.browse(cr, uid, ids):
            criteria = [('account_voucher_id','=',voucher.id)]
            import_ids = obj_detail.search(cr, uid, criteria)
            result += import_ids

        return result

    def _get_bank_payment(self, cr, uid, ids, context=None):
        result = []
        obj_voucher = self.pool.get('account.bank_payment')
        obj_detail = self.pool.get('account.import_bank_statement_detail')

        for voucher in obj_voucher.browse(cr, uid, ids):
            criteria = [('account_voucher_id','=',voucher.id)]
            import_ids = obj_detail.search(cr, uid, criteria)
            result += import_ids

        return result

    def _get_bank_receipt(self, cr, uid, ids, context=None):
        result = []
        obj_voucher = self.pool.get('account.bank_receipt')
        obj_detail = self.pool.get('account.import_bank_statement_detail')

        for voucher in obj_voucher.browse(cr, uid, ids):
            criteria = [('account_voucher_id','=',voucher.id)]
            import_ids = obj_detail.search(cr, uid, criteria)
            result += import_ids

        return result

    _columns =  {
                'import_bank_statement_id' : fields.many2one(string='Import Bank Statement', obj='account.import_bank_statement', ondelete='cascade'),
                'name' : fields.char(string='Description', size=255, required=True),
                'date' : fields.date(string='Date', required=True), #TODO: Help tips
                'transaction_type' : fields.selection(string='Type', selection=[('dr','DR'),('cr','CR')], required=True), #TODO: Help tips
                'amount' : fields.float(string='Amount', required=True), #TODO: Help tips
                'account_voucher_id' : fields.many2one(string='Bank Receipt/Payment', obj='account.voucher'), #TODO: Help tips

                'state' : fields.function(string='Voucher State', fnct=function_state, type='selection', selection=[('unassigned','Unassigned'),('draft','Draft'),('confirm','Waiting For Approval'),('approve','Ready To Process'),('proforma','Pro-Forma'),('posted','Posted'),('cancel','Cancelled')], 
                    store={
                            'account.import_bank_statement_detail' : (lambda self, cr, uid, ids, c={}:ids, ['account_voucher_id'], 10),
                            'account.import_bank_statement' : (_get_import_bank_statement, None, 10),
                            'acount.voucher' : (_get_account_voucher, None, 10),
                            'account.bank_payment' : (_get_bank_payment, None, 10),
                            'account.bank_receipt' : (_get_bank_receipt, None, 10),
                            }
                        , method=True, readonly=True),
                }

    def button_unassign_voucher(self, cr, uid, ids, context={}):
        for id in ids:
            if not self._unassign_voucher(cr, uid, id):
                return False

        return True

    def _create_account_voucher(self, cr, uid, id):
        obj_account_voucher = self.pool.get('account.voucher')
        obj_detail_voucher = self.pool.get('account.voucher.line')

        detail = self.browse(cr, uid, [id])[0]

        if detail.account_voucher_id:
            return True

        dict_voucher = self._prepare_voucher_header(cr, uid, id)

        account_voucher_id = obj_account_voucher.create(cr, uid, dict_voucher)

        dict_detail = self._prepare_voucher_detail(cr, uid, id, account_voucher_id)

        detail_id = obj_detail_voucher.create(cr, uid, dict_detail)

        self.write(cr, uid, [id], {'account_voucher_id' : account_voucher_id})

        return True


    def _prepare_voucher_header(self, cr, uid, id):
        obj_data = self.pool.get('ir.model.data')

        detail = self.browse(cr, uid, [id])[0]
        header = detail.import_bank_statement_id

        if detail.transaction_type == 'dr':
            account_id = header.journal_id.default_debit_account_id.id
            voucher_type = 'receipt'
            voucher_type_id = obj_data.get_object_reference(cr, uid, 'ar_account', 'data_voucherType_bankReceive')[1]
        else:
            account_id = header.journal_id.default_credit_account_id.id
            voucher_type = 'payment'
            voucher_type_id = obj_data.get_object_reference(cr, uid, 'ar_account', 'data_voucherType_bankPayment')[1]

        
        
        dict_result =   {
                        'name' : detail.name,
                        'date' : detail.date,
                        'period_id' : header.period_id.id, #TODO
                        'journal_id' : header.journal_id.id,
                        'company_id' : header.company_id.id, #TODO,
                        'account_id' : account_id, #TODO,
                        'type' : voucher_type, #TODO,
                        'voucher_type_id' : voucher_type_id, #TODO
                        'partner_id' : False,
                        'payment_method' : 'bank_transfer',
                        'amount' : abs(detail.amount), #TODO
                        }

        return dict_result

    def _prepare_voucher_detail(self, cr, uid, id, voucher_id):
        detail = self.browse(cr, uid, [id])[0]

        if detail.transaction_type == 'dr':
            transaction_type = 'cr'
        else:
            transaction_type = 'dr'

        dict_result =   {
                        'voucher_id' : voucher_id,
                        'name' : detail.name,
                        'type' : transaction_type,
                        'partner_id' : False,
                        'account_id' : detail.import_bank_statement_id.temporary_account_id.id,
                        'amount' : abs(detail.amount), #TODO
                        }

        return dict_result

    def _unassign_voucher(self, cr, uid, id):
        detail = self.browse(cr, uid, [id])[0]

        if not detail.account_voucher_id:
            raise osv.except_osv(_('Warning!'),_('No voucher assigned'))
            return False

        if detail.account_voucher_id.state != 'cancel':
            raise osv.except_osv(_('Warning!'),_('Please cancel the voucher first'))
            return False

        self.write(cr, uid, [id], {'account_voucher_id' : False})

        return True

import_bank_statement_detail()


