# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv
from openerp import netsvc
from datetime import datetime
from openerp.tools.translate import _

SELECTION_STATE =   [
                    ('draft','Draft'),
                    ('confirmed','Waiting For Approval'),
                    ('ready','Approved'),
                    ('done','Done'),
                    ('cancelled','Cancelled'),
                    ]

class imprest_replenish(osv.osv):
    _name = 'account.imprest_replenish'
    _description = 'Imprest Replenish'
    _inherit = ['mail.thread']

    def _default_name(self, cr, uid, context={}):
        return '/'

    def _default_state(self, cr, uid, context={}):
        return 'draft'

    def _default_date(self, cr, uid, context={}):
        return datetime.now().strftime('%Y-%m-%d')

    def _default_company_id(self, cr, uid, context={}):
        obj_user = self.pool.get('res.users')
        user = obj_user.browse(cr, uid, [uid])[0]

        return user.company_id and user.company_id.id or False

    def function_amount(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        for imprest in self.browse(cr, uid, ids):
            res[imprest.id] =   {
                                'amount_budget' : imprest.journal_id.petty_cash_account_id.balance or 0.0,
                                'total_in' : 0.0,
                                'total_out' : 0.0,
                                'total_replenish' : 0.0,
                                'current_balance' : 0.0,
                                }



            for detail in imprest.voucher_ids:
                res[imprest.id]['total_in'] += detail.total_cr
                res[imprest.id]['total_out'] += detail.total_dr
                res[imprest.id]['total_replenish'] += (detail.total_cr - detail.total_dr)

            
            res[imprest.id]['current_balance'] = res[imprest.id]['amount_budget'] + res[imprest.id]['total_replenish']
            res[imprest.id]['total_replenish'] = abs(res[imprest.id]['total_replenish'])
            if res[imprest.id]['amount_budget'] < res[imprest.id]['current_balance']:
                res[imprest.id]['total_replenish'] = 0.0
            else:
                res[imprest.id]['total_replenish'] = res[imprest.id]['amount_budget'] - res[imprest.id]['current_balance']

        return res

    def function_currency_id(self, cr, uid, ids, field_name, args, context=None):
        res = {}
        for imprest in self.browse(cr, uid, ids):
            if imprest.journal_id.currency:
                res[imprest.id] = imprest.journal_id.currency.id
            else:
                res[imprest.id] = imprest.company_id.currency_id.id

        return res



    _columns =  {
                'name' : fields.char(string='# Replenish', size=50, readonly=True, required=True),
                'date' : fields.date(string='Date', required=True, readonly=False,
                                    states={'draft':[('readonly',False)]}),
                'company_id' : fields.many2one(string='Company', obj='res.company', required=True, readonly=True,
                                                states={'draft':[('readonly',False)]}),
                'period_id' : fields.many2one(string='Period', obj='account.period', required=True, readonly=True,
                                                states={'draft':[('readonly',False)]}),
                'journal_id' : fields.many2one(string='Journal', obj='account.journal', 
                                                domain=[('cash_method','=','imprest')], required=True,
                                                readonly=True, states={'draft':[('readonly',False)]}),
                'currency_id' : fields.function(string='Currency', fnct=function_currency_id, type='many2one', 
                                                obj='res.currency', store=False, method=True),
                'description' : fields.text(string='Extra Information'),
                'amount_budget' : fields.function(string='Budget', fnct=function_amount, type='float', 
                                                    help='Petty cash budget',
                                                    method=True, store=False, multi='amount'),
                'total_in' : fields.function(string='Total Receipt', fnct=function_amount, type='float', 
                                                method=True, store=False, multi='amount'),
                'total_out' : fields.function(string='Total Payment', fnct=function_amount, type='float', 
                                                method=True, store=False, multi='amount'),
                'total_replenish' : fields.function(string='Total Replenish', fnct=function_amount, 
                                                    type='float', method=True, store=False, multi='amount'),
                'current_balance' : fields.function(string='Current Balance', fnct=function_amount, 
                                                    type='float', method=True, store=False, multi='amount'),
                'move_id': fields.many2one(string='# Entry', obj='account.move', readonly=True),
                'move_line_id' : fields.many2one(string='# Move', obj='account.move.line', readonly=True),
                'replenish_move_id' : fields.many2one(string='Replenish Move', obj='account.move.line', 
                    ondelete='restrict'),
                'state' : fields.selection(string='State', selection=SELECTION_STATE, required=True, readonly=True),
                'voucher_ids' : fields.one2many(string='Voucher', obj='account.voucher', fields_id='imprest_replenish_id',
                                                readonly=True, states={'draft':[('readonly',False)]}),
                }

    _defaults = {
                'name' : _default_name,
                'state' : _default_state,
                'date' : _default_date,
                'company_id' : _default_company_id,
                }

    def search(self, cr, uid, args, offset=0, limit=None, order=None, context={}, count=False):
        if context is None:
            context = {}

        if context.get('restrict_group', False):
            obj_journal = self.pool.get('account.journal')
            journal_ids = obj_journal.search(cr, uid, [], context={'restrict_group':1})
            args.append(('journal_id','in',journal_ids))

        ids = super(imprest_replenish, self).search(cr, uid, args, offset, limit, order, context, count)

        return ids


    def workflow_action_confirm(self, cr, uid, ids, context={}):
        for id in ids:
            if not self._create_sequence(cr, uid, id):
                return False

            self.write(cr, uid, [id], {'state' : 'confirmed'})

        return True

    def workflow_action_approve(self, cr, uid, ids, context={}):
        for id in ids:

            self.write(cr, uid, [id], {'state' : 'ready'})

        return True

    def workflow_action_done(self, cr, uid, ids, context={}):
        for id in ids:
            if not self._check_replenish_move(cr, uid, id):
                return False

            if not self._create_account_move(cr, uid, id):
                return False

            self.write(cr, uid, [id], {'state' : 'done'})

        return True

    def workflow_action_cancel(self, cr, uid, ids, context={}):
        for id in ids:
            if not self._delete_account_move(cr, uid, id):
                return False

            self.write(cr, uid, [id], {'state' : 'cancelled'})

        return True

    def button_cancel(self, cr, uid, ids, context={}):
        wkf_service = netsvc.LocalService('workflow')
        for id in ids:
            if not self._delete_workflow_instance(cr, uid, id):
                return False

            if not self._create_workflow_instance(cr, uid, id):
                return False

            wkf_service.trg_validate(uid, 'account.imprest_replenish', id, 'button_cancel', cr)

        return True

    def button_set_to_draft(self, cr, uid, ids, context={}):
        for id in ids:
            if not self._delete_workflow_instance(cr, uid, id):
                return False

            if not self._create_workflow_instance(cr, uid, id):
                return False

            self.write(cr, uid, [id], {'state' : 'draft'})

        return True

    def button_reset_number(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        for id in ids:
            if not self._reset_number(cr, uid, id):
                return False

        return True

    def _check_replenish_move(self, cr, uid, id):
        imprest = self.browse(cr, uid, [id])[0]

        if not imprest.replenish_move_id:
            raise osv.except_osv(_('Warning!'),_('No replenish move'))
            return False

        return True

    def _reset_number(self, cr, uid, id):
        self.write(cr, uid, [id], {'name' : '/'})
        return True

    def _create_account_move(self, cr, uid, id):
        imprest = self.browse(cr, uid, [id])[0]
        obj_move = self.pool.get('account.move')
        obj_move_line = self.pool.get('account.move.line')
        obj_reconcile = self.pool.get('account.move.reconcile')
        list_move = []

        res =   {
                'name' : imprest.name,
                'journal_id' : imprest.journal_id.id,
                'date' : imprest.date,
                'period_id' : imprest.period_id.id,
                }

        move_id = obj_move.create(cr, uid, res)

        res_header =    {
                        'move_id' : move_id,
                        'name' : 'Refill',
                        'account_id' : imprest.replenish_move_id.account_id.id,
                        'debit' : 0.0,
                        'credit' : imprest.total_replenish,
                        }

        move_line_id = obj_move_line.create(cr, uid, res_header)
        self.write(cr, uid, [id], {'move_id' : move_id, 'move_line_id' : move_line_id})

        for voucher in imprest.voucher_ids:
            for detail in voucher.line_ids:
                res_detail =    {
                                'move_id' : move_id,
                                'name' : detail.name,
                                'account_id' : detail.account_id.id,
                                'debit' : detail.type == 'dr' and (detail.amount_in_company_currency - detail.amount_tax_in_company_currency) or 0.0,
                                'credit' : detail.type == 'cr' and (detail.amount_in_company_currency - detail.amount_tax_in_company_currency) or 0.0,
                                'partner_id' : detail.partner_id and detail.partner_id.id or False,
                                'analytic_account_id' : detail.account_analytic_id and detail.account_analytic_id.id or False,
                                'analytics_id' : detail.analytic_plan_id and detail.analytic_plan_id.id or False,
                                'direct_analytic_ids' : [],
                                'indirect_analytic_ids' : [],
                                }

                if detail.direct_analytic_ids:
                    for direct in detail.direct_analytic_ids:
                        value_direct =  {
                                        'cost_object_id' : direct.cost_object_id.id,
                                        'analytic_account_id' : direct.analytic_account_id.id,
                                        }

                        res_detail['direct_analytic_ids'].append((0,0,value_direct))


                if detail.indirect_analytic_ids:
                    for indirect in detail.indirect_analytic_ids:
                        value_indirect =    {
                                            'cost_object_id' : direct.cost_object_id.id,
                                            'analytic_distribution_id' : direct.analytic_distribution_id.id,
                                            }

                        res_detail['indirect_analytic_ids'].append((0,0,value_indirect))

                move_detail_id = obj_move_line.create(cr, uid, res_detail)

                if detail.tax_line_ids:
                    for tax in detail.tax_line_ids:
                        debit = 0.0
                        credit = 0.0

                        if detail.type == 'dr':
                            if tax.function_tax_amount_in_company_currency > 0:
                                debit = abs(tax.function_tax_amount_in_company_currency)
                            elif tax.function_tax_amount_in_company_currency < 0:
                                credit = abs(tax.function_tax_amount_in_company_currency)
                        elif detail.type == 'cr':
                            if tax.function_tax_amount_in_company_currency < 0:
                                credit = abs(tax.function_tax_amount_in_company_currency)
                            elif tax.function_tax_amount_in_company_currency > 0:
                                debit = abs(tax.function_tax_amount_in_company_currency)

                        res_tax =   {
                                    'move_id' : move_id,
                                    'name' : tax.name,
                                    'account_id' : tax.account_id.id,
                                    'debit' : debit,
                                    'credit' : credit,
                                    'direct_analytic_ids' : [],
                                    'indirect_analytic_ids' : [],
                                    }

                        if detail.direct_analytic_ids:
                            for direct in detail.direct_analytic_ids:
                                value_direct =  {
                                                'cost_object_id' : direct.cost_object_id.id,
                                                'analytic_account_id' : direct.analytic_account_id.id,
                                                }

                                res_tax['direct_analytic_ids'].append((0,0,value_direct))


                        if detail.indirect_analytic_ids:
                            for indirect in detail.indirect_analytic_ids:
                                value_indirect =    {
                                                    'cost_object_id' : direct.cost_object_id.id,
                                                    'analytic_distribution_id' : direct.analytic_distribution_id.id,
                                                    }

                                res_tax['indirect_analytic_ids'].append((0,0,value_indirect))

                        obj_move_line.create(cr, uid, res_tax)

                if detail.move_line_id:
                    line_ids = []
                    reconcile_id = False

                    if detail.move_line_id.reconcile_partial_id.partial_ids:
                        reconcile_id = detail.move_line_id.reconcile_partial_id.id
                        # for partial_id in detail.move_line_id.reconcile_partial_id.partial_ids:
                            # line_ids.append(partial_id.id)

                    if detail.move_line_id.reconcile_id.line_id:
                        reconcile_id = detail.move_line_id.reconcile_id.id
                        # for full_id in detail.move_line_id.reconcile_id.line_id:
                        #     line_ids.append(full_id.id)

                    if reconcile_id:
                        # line_ids.remove(detail.move_line_id.id)
                        obj_reconcile.unlink(cr, uid, [reconcile_id])
                        line_ids.append(move_detail_id)
                        line_ids.append(detail.move_line_id.id)
                        list_move.append(line_ids)


        # if len(list_move) > 0:
        #     for move_pair in list_move:
        #         if len(move_pair) >= 2:
        #             obj_move_line.reconcile_partial(cr, uid, move_pair)

        obj_move.post(cr, uid, [move_id])

        return True


    def _create_sequence(self, cr, uid, id):
        obj_sequence = self.pool.get('ir.sequence')

        imprest = self.browse(cr, uid, [id])[0]

        if imprest.name == '/':
            # context = {'fiscalyear_id' : imprest.period_id.fiscalyear_id.id}
            context = {'source_date' : imprest.date}
            if not imprest.journal_id.source_journal_id:
                raise osv.except_osv(_('Warning!'),_('No source journal defined'))

            name = obj_sequence.next_by_id(cr, uid, imprest.journal_id.source_journal_id.sequence_id.id, context=context)
            self.write(cr, uid, [id], {'name' : name})


        return True

    def _delete_account_move(self, cr, uid, id):
        obj_move = self.pool.get('account.move')
        imprest = self.browse(cr, uid, [id])[0]
        move_id = imprest.move_id and imprest.move_id.id or False

        self.write(cr, uid, [id], {'move_id' : False, 'move_line_id' : False})

        if move_id:
            obj_move.button_cancel(cr, uid, [move_id])
            obj_move.unlink(cr, uid, [move_id])

        return True

    def _delete_workflow_instance(self, cr, uid, id):
        wkf_service = netsvc.LocalService('workflow')
        wkf_service.trg_delete(uid, 'account.imprest_replenish', id, cr)

        return True

    def _create_workflow_instance(self, cr, uid, id):
        wkf_service = netsvc.LocalService('workflow')
        wkf_service.trg_create(uid, 'account.imprest_replenish', id, cr)

        return True

    def onchange_date(self, cr, uid, ids, date):
        value = {}
        domain = {}
        warning = {}

        obj_period = self.pool.get('account.period')

        period_ids = obj_period.find(cr, uid, date)

        if period_ids:
            period_id = period_ids[0]
            value.update({'period_id' : period_id})

        return {'value' : value, 'domain' : domain, 'warning' : warning}





imprest_replenish()




