# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv
import openerp.addons.decimal_precision as dp
from openerp.tools import float_round
from openerp.tools.safe_eval import safe_eval as eval
from openerp.tools.translate import _
import math

class voucher_line_tax(osv.osv):
    _name = 'account.voucher_line_tax'
    _description = 'Voucher Line Tax'

    def _default_manual(self, cr, uid, context=None):
        return True

    def _default_sequence(self, cr, uid, context=None):
        return 5

    def _compute_balance(self, cr, uid, ids, field_name, args, context=None):
        obj_currency = self.pool.get('res.currency')
        obj_tax = self.pool.get('account.tax')

        if context is None:
            context = {}

        res = {}

        for tax in self.browse(cr, uid, ids):

            res[tax.id] =   {
                            'function_base_amount' : 0.0,
                            'function_rate_conversion_method' : 'indirect',
                            'function_exchange_rate' : 0.0,
                            'function_base_amount_in_company_currency' : 0.0,
                            'function_tax_amount_in_company_currency' : 0.0,
                            }

            vline = tax.voucher_line_id or False
            move_line = vline.move_line_id or False
            

            ctx = context.copy()
            ctx.update({'date': vline.voucher_id.date})

            company_currency_id = vline.voucher_id.company_id.currency_id.id
            voucher_currency_id = vline.voucher_id.journal_id.currency and vline.voucher_id.journal_id.currency.id \
                                    or company_currency_id

            voucher_rate = vline.voucher_id.exchange_rate

            if move_line:
                move_rate = obj_currency.compute(cr, uid, voucher_currency_id, company_currency_id, \
                        1.0, context={'date' : move_line.date})



            if tax.base_amount_computation_method == 'system':
                res[tax.id]['function_base_amount'] = vline.amount
                if tax.tax_id.base_amount_rounding_method == 'currency':
                    res[tax.id]['function_base_amount'] = float_round(res[tax.id]['function_base_amount'], precision_rounding=tax.company_currency_id.rounding)
                else:
                    localdict = {
                                'self' : self,
                                'cr' : cr,
                                'uid' : uid,
                                'base_amount' : res[tax.id]['function_base_amount'],
                                'result' : res[tax.id]['function_base_amount'],
                                'float' : float,
                                'int' : int,
                                'math' : math,
                                'float_round' : float_round,
                                }
                    eval(tax.tax_id.base_amount_rounding_code, localdict, mode='exec', nocopy=True)
                    res[tax.id]['function_base_amount'] = localdict['result']
            else:
                res[tax.id]['function_base_amount'] = tax.base_amount

            if tax.rate_options == 'system':
                res[tax.id]['function_rate_conversion_method'] = vline.voucher_id.company_id.exchange_rate_method
                if move_line:
                    res[tax.id]['function_exchange_rate'] = move_rate
                else:
                    res[tax.id]['function_exchange_rate'] = voucher_rate
            else:
                res[tax.id]['function_rate_conversion_method'] = tax.rate_conversion_method
                res[tax.id]['function_exchange_rate'] = tax.exchange_rate


            if tax.base_amount_in_company_currency_computation_method == 'system':
                if res[tax.id]['function_rate_conversion_method'] == 'direct':
                    res[tax.id]['function_base_amount_in_company_currency'] = \
                            (res[tax.id]['function_base_amount'] * res[tax.id]['function_exchange_rate'])
                else:
                    res[tax.id]['function_base_amount_in_company_currency']  = \
                            res[tax.id]['function_base_amount'] * (1.0 / res[tax.id]['function_exchange_rate'])

                if tax.tax_id.base_amount_in_company_currency_rounding_method == 'currency':
                    res[tax.id]['function_base_amount_in_company_currency'] = \
                            float_round(res[tax.id]['function_base_amount_in_company_currency'], \
                            precision_rounding=tax.company_currency_id.rounding)
                else:
                    localdict = {
                                'self' : self,
                                'cr' : cr,
                                'uid' : uid,
                                'base_amount_in_company_currency' : res[tax.id]['function_base_amount_in_company_currency'],
                                'result' : res[tax.id]['function_base_amount_in_company_currency'],
                                'float' : float,
                                'int' : int,
                                'math' : math,
                                'float_round' : float_round,
                                }
                    eval(tax.tax_id.base_amount_in_company_currency_rounding_code, localdict, mode='exec', nocopy=True)
                    res[tax.id]['function_base_amount_in_company_currency'] = localdict['result']
            else:
                res[tax.id]['function_base_amount_in_company_currency'] = tax.base_amount_in_company_currency

            if tax.tax_amount_in_company_currency_computation_method == 'system':
                tax_computation = obj_tax._compute(cr, uid, [tax.tax_id], res[tax.id]['function_base_amount_in_company_currency'],1.0)[0]['amount']
                res[tax.id]['function_tax_amount_in_company_currency'] = tax_computation

                if tax.tax_id.tax_amount_in_company_currency_rounding_method == 'currency':
                    res[tax.id]['function_tax_amount_in_company_currency'] = \
                            float_round(res[tax.id]['function_tax_amount_in_company_currency'], \
                            precision_rounding=tax.company_currency_id.rounding)
                else:
                    localdict = {
                                'self' : self,
                                'cr' : cr,
                                'uid' : uid,
                                'tax_amount_in_company_currency' : res[tax.id]['function_tax_amount_in_company_currency'],
                                'result' : res[tax.id]['function_tax_amount_in_company_currency'],
                                'float' : float,
                                'int' : int,
                                'math' : math,
                                'float_round' : float_round,
                                }
                    eval(tax.tax_id.tax_amount_in_company_currency_rounding_code, localdict, mode='exec', nocopy=True)
                    res[tax.id]['function_tax_amount_in_company_currency'] = localdict['result']
            else:
                res[tax.id]['function_tax_amount_in_company_currency'] = tax.tax_amount_in_company_currency

        return res

    def _default_rate_options(self, cr, uid, context=None):
        return 'system'

    def _default_rate_conversion_method(self, cr, uid, context=None):
        return 'indirect'

    def _default_base_amount_computation_method(self, cr, uid, context={}):
        return 'system'

    def _default_base_amount_in_company_currency_computation_method(self, cr, uid, context={}):
        return 'system'

    def _default_tax_amount_in_company_currency_computation_method(self, cr, uid, context={}):
        return 'system'

    _columns =  {
                'voucher_line_id' : fields.many2one(string='Voucher Line', obj='account.voucher.line',
                    required=True, ondelete='cascade'),
                'name' : fields.char(string='Description', size=255, required=True),
                'tax_id' : fields.many2one(string='Tax', obj='account.tax', required=True),
                'account_id' : fields.many2one(string='Account', obj='account.account', required=True),
                'tax_rate' : fields.float(string='Tax Rate', digits_compute=dp.get_precision('Account')),
                'voucher_currency_id' : fields.related('voucher_line_id','voucher_id','currency_id',
                    type='many2one', relation='res.currency', string='Voucher Currency'),
                'company_currency_id' : fields.related('voucher_line_id','voucher_id','payment_rate_currency_id',
                    type='many2one', relation='res.currency', string='Company Currency'),

                #BASE AMOUNT
                'base_amount' : fields.float(string='Base Amount', digits_compute=dp.get_precision('Account')),
                'base_amount_computation_method' : fields.selection(string='Base Amount Computation Method',
                    selection=[('manual','Manual'),('system','System')], required=True),
                'function_base_amount' : fields.function(fnct=_compute_balance, multi='dc', type='float', 
                    string='Function Base Amount', method=True, store=False, digits_compute=dp.get_precision('Account')),


                'base_amount_in_company_currency_computation_method' : fields.selection(string='Base Amount In Company Currency Computation Method',
                    selection=[('manual','Manual'),('system','System')], required=True),
                'base_amount_in_company_currency' : fields.float(string='Base Amount In Company Currency',
                    digits_compute=dp.get_precision('Account')),
                'rate_options' : fields.selection(string='Exchange Rate Options', 
                    selection=[('manual','Manual'),('system','System')], required=True),
                'rate_conversion_method' : fields.selection(string='Rate Convertion Method', 
                    selection=[('direct','Direct'),('indirect','Indirect')], required=True),
                'exchange_rate' : fields.float(string='Exchange Rate', digits_compute=dp.get_precision('Account')),
                'function_rate_conversion_method' : fields.function(string='Rate Conversion Method', 
                    fnct=_compute_balance, selection=[('direct','Direct'),('indirect','Indirect')], 
                    type='selection', method=True, store=False, multi='dc'),
                'function_exchange_rate' : fields.function(fnct=_compute_balance, multi='dc', type='float', 
                    string='Exchange Rate', method=True, store=False, digits_compute=dp.get_precision('Account')),
                'function_base_amount_in_company_currency' : fields.function(fnct=_compute_balance, multi='dc', type='float', 
                    string='Base Amount In Company Currency', method=True, store=False, digits_compute=dp.get_precision('Account')),

                'tax_amount_in_company_currency_computation_method' : fields.selection(string='Tax Amount Computation Method',
                    selection=[('manual','Manual'),('system','System')], required=True),
                'tax_amount_in_company_currency' : fields.float(string='Tax Amount In Company Currency',
                    digits_compute=dp.get_precision('Account')),
                'function_tax_amount_in_company_currency' : fields.function(fnct=_compute_balance, multi='dc', type='float', 
                    string='Tax Amount In Company Currency', method=True, store=False, digits_compute=dp.get_precision('Account')),

                'manual' : fields.boolean(string='Manual', readonly=True),
                'sequence' : fields.integer(string='Sequence'),
                }

    _defaults = {
                'manual' : _default_manual,
                'rate_options' : _default_rate_options,
                'rate_conversion_method' : _default_rate_conversion_method,
                'base_amount_computation_method' : _default_base_amount_computation_method,
                'base_amount_in_company_currency_computation_method' : _default_base_amount_in_company_currency_computation_method,
                'tax_amount_in_company_currency_computation_method' : _default_tax_amount_in_company_currency_computation_method,
                'sequence' : _default_sequence,
                }


    def _move_item_debit_credit_amount(self, cr, uid, type='dr', amount=0.0, currency_id=False, amount_currency=0.0):
        result =    {
                    'debit' : 0.0,
                    'credit' : 0.0,
                    'currency_id' : currency_id,
                    'amount_currency' : 0.0,
                    }

        if type == 'dr':
            if amount > 0:
                result['debit'] = abs(amount)
            else:
                result['credit'] = abs(amount)

            result['amount_currency'] = currency_id and abs(amount_currency) or 0.0
        elif type == 'cr':
            if amount > 0:
                result['credit'] = abs(amount)
            else:
                result['debit'] = abs(amount)

            result['amount_currency'] = currency_id and -1.0 * abs(amount_currency) or 0.0

        return result

    def _prepare_move_line(self, cr, uid, id, move_id):
        tax = self.browse(cr, uid, [id])[0]
        line = tax.voucher_line_id

        debit_credit_amount = self._move_item_debit_credit_amount(cr, uid, line.type, \
                tax.function_tax_amount_in_company_currency, False, 0.0)

        result =    {
                    'name' : tax.name,
                    'debit' : debit_credit_amount['debit'],
                    'credit' : debit_credit_amount['credit'],
                    'account_id' : self._get_account(cr, uid, tax.id),
                    'move_id' : move_id,
                    'amount_currency' : debit_credit_amount['amount_currency'],
                    'currency_id' : debit_credit_amount['currency_id'],
                    'product_id' : False,
                    'quantity' : False,
                    'product_uom_id' : False,
                    'partner_id' : line.partner_id and line.partner_id.id or False,
                    }

        return result

    def onchange_tax_id(self, cr, uid, ids, tax_id, voucher_type=False):
        domain = {}
        value = {}
        warning = {}
        account_id = False

        obj_tax = self.pool.get('account.tax')

        if tax_id and voucher_type:
            tax = obj_tax.browse(cr, uid, [tax_id])[0]

            if voucher_type == 'payment':
                account_id = tax.account_payment_id and tax.account_payment_id.id or False,
            elif voucher_type == 'receipt':
                account_id = tax.account_receipt_id and tax.account_receipt_id.id or False,
            

            value = {
                    'name' : tax.name,
                    'account_id' : account_id,
                    }

        return {'value' : value, 'domain' : domain, 'warning' : warning}

    def _get_account(self, cr, uid, id):
        tax = self.browse(cr, uid, [id])[0]
        voucher_type = tax.voucher_line_id.voucher_id.voucher_type_id

        if voucher_type.detail_account == 'voucher_detail_account':
            return tax.account_id.id
        elif voucher_type.detail_account == 'journal_temporary_income_account':
            if not tax.tax_id.account_imprest_receipt_id:
                raise osv.except_osv(_('Warning!'),_('No temporary account defined'))

            return tax.tax_id.account_imprest_receipt_id.id
        elif voucher_type.detail_account == 'journal_temporary_expense_account':
            if not tax.tax_id.account_imprest_payment_id:
                raise osv.except_osv(_('Warning!'),_('No temporary account defined'))

            return tax.tax_id.account_imprest_payment_id.id
        else:
            return False
voucher_line_tax()




