# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv
from openerp.tools.translate import _

DEFAULT_HEADER_TYPE = [
    ('sale', 'Sale'),
    ('purchase', 'Purchase'),
    ('payment', 'Payment'),
    ('receipt', 'Receipt'),
]

DEFAULT_DETAIL_TYPE_SELECTION = [
    ('dr', 'Debit'),
    ('cr', 'Credit'),
]

SELECTION_HEADER_ACCOUNT =  [
                            ('journal_default_debit','Journal Default Debit Account'),
                            ('journal_default_credit','Journal Default Credit Account'),
                            ('journal_temporary_cash_account', 'Journal Temporary Cash Account'),
                            ]

SELECTION_DETAIL_ACCOUNT =  [
                            ('voucher_detail_account','Voucher Detail Account'),
                            ('journal_temporary_expense_account','Journal Temporary Expense Account'),
                            ('journal_temporary_income_account','Journal Temporary Income Account'),
                            ]


class voucher_type(osv.osv):
    _name = 'account.voucher_type'
    _description = 'Voucher Type'
    
    def default_check_total(self, cr, uid, context={}):
        return True
        
    def default_active(self, cr, uid, context={}):
        return True     


    def _default_sequence_used(self, cr, uid, context={}):
        return 'primary'

    def _default_allow_negative(self, cr, uid, context={}):
        return False

    def _default_header_allow_negative(self, cr, uid, context={}):
        return False

    _columns =  {
                'kode' : fields.char(string='# Kode', size=100, required=True),
                'name' : fields.char(string='Name', size=100, required=True),
                'active' : fields.boolean(string='Active'),
                'description' : fields.text(string='Description'),
                'default_header_type' : fields.selection(selection=DEFAULT_HEADER_TYPE, string='Default Header Type'),
                'default_detail_type_selection' : fields.selection(selection=DEFAULT_DETAIL_TYPE_SELECTION, string='Default Detail Type Selection'),
                'allowed_journal_ids' : fields.many2many(obj='account.journal', rel='account_voucher_type_account_journal_rel', id1='voucher_type_id', id2='account_journal_id'),   
                'model_name' : fields.char(string='Model Name', size=100),
                'model_view_form' : fields.char(string='Model View Form', size=100),        
                'modul_origin' : fields.char(string='Modul Origin', size=100),  
                'check_total' : fields.boolean(string='Check For Total'),
                'check_debit_credit' : fields.boolean(string='Check Debit Credit'),
                'is_imprest' : fields.boolean(string='Is Imprest Voucher'),
                'allow_negative' : fields.boolean(string='Allow Negative Value On Detail'),
                'header_allow_negative' : fields.boolean(string='Allow Negative Value On Header'),
                'account_restricted_ids' : fields.one2many(string='Restricted Account', obj='account.voucher_type_account_restriction', fields_id='voucher_type_id'),
                'journal_restriction_ids' : fields.one2many(string='Journal Restriction', obj='account.voucher_type_journal_restriction', fields_id='voucher_type_id'),

                'create_header_item' : fields.boolean(string='Create Header Item'),
                'header_item_type' : fields.selection(string='DR/CR?', selection=[('dr','Debit'),('cr','Credit')]),
                'header_account' : fields.selection(string='Use Account From', selection=SELECTION_HEADER_ACCOUNT),

                'detail_item_type' : fields.selection(string='DR/CR?', selection=[('dr','Debit'),('cr','Credit')]),
                'detail_account' : fields.selection(string='Use Account From', selection=SELECTION_DETAIL_ACCOUNT),

                'sequence_used' : fields.selection(string='Sequence Used',
                                        selection=[('primary','Primary Sequence'),('in','In Sequence'),('out','Our Sequence')],
                                        required=True),
                }
                
    _defaults = {
                'active' : default_active,
                'check_total' : default_check_total,
                'sequence_used' : _default_sequence_used,
                'allow_negative' : _default_allow_negative,
                'header_allow_negative' : _default_header_allow_negative,
                }

voucher_type()

class voucher_type_account_restriction(osv.osv):
    _name = 'account.voucher_type_account_restriction'
    _description = 'Voucher Type Account Restriction'

    _columns =  {
                'voucher_type_id' : fields.many2one(string='Voucher Type', obj='account.voucher_type', required=True, ondelete='restrict'),
                'journal_id' : fields.many2one(string='Journal', obj='account.journal', required=True),
                'account_restricted_ids' : fields.many2many(string='Restricted Account', obj='account.account', rel='rel_voucher_type_acc_restriction', id1='restrict_id', id2='account_id', domain=[('type','not in',['view','consolidation','close'])]),
                }

    _sql_constraints =   [
                        ('voucher_type_journal_unique','unique(voucher_type_id, journal_id)',_('Duplicate journal')),
                        ]



voucher_type_account_restriction()

class voucher_type_journal_restriction(osv.osv):
    _name = 'account.voucher_type_journal_restriction'
    _description = 'Voucher Type Journal Restriction'

    _columns =  {
                'voucher_type_id' : fields.many2one(string='Voucher Type', obj='account.voucher_type', required=True, ondelete='restrict'),
                'journal_id' : fields.many2one(string='Journal', obj='account.journal', ondelete='restrict', required=True),
                'group_ids' : fields.many2many(string='Allowed Groups', obj='res.groups', rel='rel_vouch_type_journal_res_2_group', id1='restriction_id', id2='group_id'),
                }


voucher_type_journal_restriction()




