# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from openerp.osv import fields, osv
from datetime import datetime, date
import calendar
from dateutil.relativedelta import relativedelta
import openerp.addons.decimal_precision as dp
import math
import logging


_logger = logging.getLogger(__name__)

class account_asset_asset(osv.osv):
    _name = 'account.asset.asset'
    _inherit = ['account.asset.asset','mail.thread']

    def _default_state(self, cr, uid, context={}):
        return 'draft'

    def _default_first_depreciation_date(self, cr, uid, context={}):
        return datetime.today().strftime('%Y-%m-%d')

    def _function_compute_all(self, cr, uid, ids, field_name, args, context=None):
        if context is None:
            context = {}

        res = {}

        if not context.get('date_check', False):
            date_check = datetime.today().strftime('%Y-%m-%d')
        else:
            date_check = context.get('date_check', False)


        for asset in self.browse(cr, uid, ids):
            res[asset.id] = {
                            'monthly_depreciation' : 0.0,
                            'depreciation_amount' : 0.0,
                            'last_depreciation_run' : False,
                            'purchase_value' : 0.0,
                            'to_be_depreciated_amount' : 0.0,
                            'book_value' : 0.0,
                            'remaining_depreciation' : 0,
                            'asset_acquisition_analytic_line_ids' : [],
                            }

            if asset.asset_registration_status == 'existing':
                res[asset.id]['depreciation_amount'] += asset.opening_depreciation_amount
                res[asset.id]['purchase_value'] = asset.opening_asset_value
            else:
                if asset.no_acquisition_tracking:
                    res[asset.id]['purchase_value'] = asset.acquisition_value
                else:
                    if asset.purchase_date and not asset.no_acquisition_tracking and asset.analytic_account_id:
                        strSQL =    """
                                    SELECT  a.id AS id
                                    FROM    account_analytic_line AS a
                                    WHERE   a.account_id = %s AND
                                            a.date <= to_date('%s', 'YYYY-MM-DD') AND
                                            a.general_account_id = %s
                                    """ % (asset.analytic_account_id.id, asset.purchase_date, asset.category_id.account_asset_id.id)

                        cr.execute(strSQL)
                        acquisitions = cr.dictfetchall()
                        if len(acquisitions) > 0:
                            for acquisition in acquisitions:
                                res[asset.id]['asset_acquisition_analytic_line_ids'].append(acquisition['id'])


                        strSQL =    """
                                    SELECT  a.account_id AS account_id,
                                            SUM(b.debit - b.credit) AS amount
                                    FROM    account_analytic_line AS a
                                    JOIN    account_move_line AS b ON a.move_id = b.id
                                    WHERE   a.account_id = %s AND
                                            a.date <= to_date('%s', 'YYYY-MM-DD') AND
                                            a.general_account_id = %s
                                    GROUP BY    a.account_id
                                    """ % (asset.analytic_account_id.id, asset.purchase_date, asset.category_id.account_asset_id.id)

                        cr.execute(strSQL)
                        acquisitions = cr.dictfetchall()

                        if len(acquisitions) > 0:
                            res[asset.id]['purchase_value'] += acquisitions[0]['amount']


            strSQL =    """
                        SELECT  a.asset_id,
                                SUM(a.amount) AS amount,
                                COUNT(a.asset_id) AS depreciation_count
                        FROM    account_asset_depreciation_line AS a
                        WHERE   a.asset_id=%s AND
                                a.move_check = True AND
                                a.depreciation_date <= to_date('%s','YYYY-MM-DD')
                        GROUP BY    a.asset_id
                        
                        """ % (asset.id, date_check)

            cr.execute(strSQL)
            result = cr.dictfetchall()

            if len(result) > 0:
                res[asset.id]['depreciation_amount'] += result[0]['amount']
                res[asset.id]['remaining_depreciation'] = result[0]['depreciation_count']

            res[asset.id]['to_be_depreciated_amount'] = res[asset.id]['purchase_value'] - asset.salvage_value - res[asset.id]['depreciation_amount']
            res[asset.id]['book_value'] = res[asset.id]['to_be_depreciated_amount'] + asset.salvage_value

            if asset.asset_registration_status == 'existing':
                res[asset.id]['remaining_depreciation'] = asset.method_number - asset.number_of_depreciation - res[asset.id]['remaining_depreciation']
            else:
                res[asset.id]['remaining_depreciation'] = asset.method_number - res[asset.id]['remaining_depreciation']


        return res

    def _default_asset_registration_status(self, cr, uid, context={}):
        return 'new'

    def _default_method_period(self, cr, uid, context={}):
        return 1

    _columns =  {
                #COMMON
                'asset_registration_status' : fields.selection(string='Asset Registraion Status', required=True, readonly=True,
                    selection=[('new','New Asset'),('existing','Existing Asset')],
                    states={'draft':[('readonly',False)]}),
                'registration_date': fields.date(string='Registration Date', required=True, readonly=True, 
                    states={'draft':[('readonly',False)]}),
                'purchase_date': fields.date(string='In Service', required=True, readonly=True, 
                    states={'draft':[('readonly',False)]}),
                'first_depreciation_date' : fields.date(string='First Depreciation Date', required=True, readonly=True,
                    states={'draft':[('readonly',False)],'acquiring':[('readonly',False)]}),
                'analytic_account_id' : fields.many2one(string='Analytic Account', readonly=True,
                    obj='account.analytic.account',
                    states={'draft':[('readonly',False)]}),
                'state' : fields.selection(string='State', readonly=True, required=True,
                    selection=[('draft','Draft'),('acquiring','Acquiring'),('open','Active'),('disposing','Disposing'),('close','Disposed')]),
                'monthly_depreciation' : fields.function(string='Monthly Depreciation', type='float',
                    fnct=_function_compute_all, method=True, store=False, multi='all'),
                'depreciation_amount' : fields.function(string='Accumulated Depreciation', type='float',
                    fnct=_function_compute_all, method=True, store=False, multi='all'),
                'to_be_depreciated_amount' : fields.function(string='To be Depreciated Amount', type='float',
                    fnct=_function_compute_all, method=True, store=False, multi='all'),
                'last_depreciation_run' : fields.function(string='Last Depreciation Run', type='date',
                    fnct=_function_compute_all, method=True, store=False, multi='all'),
                'purchase_value' : fields.function(string='Asset Value', type='float',
                    fnct=_function_compute_all, method=True, store=False, multi='all'),
                'book_value' : fields.function(string='Book Value', type='float',
                    fnct=_function_compute_all, method=True, store=False, multi='all'),
                'remaining_depreciation' : fields.function(string='Remaining Number of Depreciation', type='integer',
                    fnct=_function_compute_all, method=True, store=False, multi='all'),

                #WILL BE DELETED
                'no_acquisition_tracking' : fields.boolean(string='No Acquisition Tracking', readonly=True,
                    states={'draft':[('readonly',False)],'acquiring':[('readonly',False)]}),

                #NEW ASSET
                'asset_acquisition_analytic_line_ids' : fields.function(string='Asset Acquisition Analytic Line',
                    type='many2many',
                    fnct=_function_compute_all,
                    relation='account.analytic.line', 
                    rel='rel_asset_2_acquisition', 
                    id1='asset_id', 
                    id2='analytic_line_id',
                    method=True,
                    store=False,
                    multi='all',
                    ),

                'asset_in_progress_analytic_line_ids' : fields.many2many(string='Asset In-Progress Analytic Line',
                    obj='account.analytic.line', rel='rel_asset_2_in_progress', id1='asset_id', id2='analytic_line_id'),

                #EXPENDITURE
                'asset_capital_expenditure_analytic_line_ids' : fields.many2many(string='Capital Expeniture Analytic Line',
                    obj='account.analytic.line', rel='rel_asset_2_capital_expenditure', id1='asset_id', id2='analytic_line_id'),

                'asset_revenue_expenditure_analytic_line_ids' : fields.many2many(string='Revenue Expeniture Analytic Line',
                    obj='account.analytic.line', rel='rel_asset_2_revenue_expenditure', id1='asset_id', id2='analytic_line_id'),

                #EXISTING ASSET
                'acquisition_value' : fields.float(string='Acquisition Value', readonly=True,
                    states={'draft':[('readonly',False)],'acquiring':[('readonly',False)]}),
                'opening_depreciation_amount' : fields.float(string='Opening Depreciation Amount'),
                'opening_asset_value' : fields.float(string='Opening Asset Value'),
                'number_of_depreciation' : fields.integer('Number of Depreciation'),

                #FILTER
                'check_date' : fields.dummy(string='As of', type='date'), 
                }

    _defaults = {
                'state' : _default_state,
                'first_depreciation_date' : _default_first_depreciation_date,
                'asset_registration_status' : _default_asset_registration_status,
                'method_period' : _default_method_period,
                }

    def workflow_action_acquiring(self, cr, uid, ids, context={}):
        for id in ids:
            self.write(cr, uid, [id], {'state' : 'acquiring'})

        return True

    def workflow_action_open(self, cr, uid, ids, context={}):
        for id in ids:
            self.write(cr, uid, [id], {'state' : 'open'})

        return True

    def workflow_action_disposing(self, cr, uid, ids, context={}):
        for id in ids:
            self.write(cr, uid, [id], {'state' : 'disposing'})

        return True

    def workflow_action_close(self, cr, uid, ids, context={}):
        for id in ids:
            self.write(cr, uid, [id], {'state' : 'close'})

        return True

    def button_set_to_draft(self, cr, uid, ids, context={}):
        for id in ids:
            self.write(cr, uid, [id], {'state' : 'draft'})

    def _create_reverse_acc_entry(self, cr, uid, id):
        return True


    def compute_depreciation_board(self, cr, uid, ids, context=None):
        obj_depreciation_line = self.pool.get('account.asset.depreciation.line')
        obj_currency = self.pool.get('res.currency')

        if context is None:
            context = {}

        for asset in self.browse(cr, uid, ids, context=context):
            if asset.value_residual == 0.0:
                continue

            criteria1 = [('asset_id', '=', asset.id), ('move_check', '=', True)]
            posted_depreciation_line_ids = obj_depreciation_line.search(cr, uid, criteria1, order='depreciation_date desc')

            criteria2 = [('asset_id', '=', asset.id), ('move_id', '=', False)]
            old_depreciation_line_ids = obj_depreciation_line.search(cr, uid, criteria2)

            if old_depreciation_line_ids:
                obj_depreciation_line.unlink(cr, uid, old_depreciation_line_ids, context=context)


            dt_first_depreciation_date = datetime.strptime(asset.first_depreciation_date, '%Y-%m-%d')

            #find out the starting point
            if (len(posted_depreciation_line_ids)>0): #depreciation already happened
                last_depreciation = obj_depreciation_line.browse(cr, uid, posted_depreciation_line_ids, context=context)[0]
                dt_last_depreciation_date = datetime.strptime(last_depreciation.depreciation_date, '%Y-%m-%d')
                dt_depreciation_date = (dt_last_depreciation_date+relativedelta(months=+asset.method_period))
                mr = calendar.monthrange(dt_depreciation_date.year, dt_depreciation_date.month)
                dt_depreciation_date = date(dt_depreciation_date.year, dt_depreciation_date.month, mr[1])
            else: #no depreciation
                dt_depreciation_date = dt_first_depreciation_date

            asset = self.browse(cr, uid, [asset.id], {'date_check' : dt_depreciation_date.strftime('%Y-%m-%d')})[0]

            amount_to_depr = residual_amount = asset.to_be_depreciated_amount
            depreciation_amount = asset.depreciation_amount

            day = dt_depreciation_date.day
            month = dt_depreciation_date.month
            year = dt_depreciation_date.year

            undone_dotation_number = asset.remaining_depreciation


            for x in range(len(posted_depreciation_line_ids), undone_dotation_number):
                i = x + 1


                amount = self._compute_depreciation(cr, uid, asset, x)

                # compute amount into company currency
                company_currency = asset.company_id.currency_id.id
                current_currency = asset.currency_id.id
                amount = obj_currency.compute(cr, uid, current_currency, company_currency, amount, context=context)
                residual_amount -= amount
                depreciation_amount += amount

                vals =  {
                        'amount': amount,
                        'asset_id': asset.id,
                        'sequence': i,
                        'name': str(asset.id) +'/' + str(i),
                        'remaining_value': residual_amount,
                        'depreciated_value': depreciation_amount,
                        'depreciation_date': dt_depreciation_date.strftime('%Y-%m-%d'),
                        }
                obj_depreciation_line.create(cr, uid, vals, context=context)



                # Considering Depr. Period as months
                dt_depreciation_date = (datetime(year, month, day) + relativedelta(months=+asset.method_period))
                mr = calendar.monthrange(dt_depreciation_date.year, dt_depreciation_date.month)
                dt_depreciation_date = date(dt_depreciation_date.year, dt_depreciation_date.month, mr[1])
                day = dt_depreciation_date.day
                month = dt_depreciation_date.month
                year = dt_depreciation_date.year
        return True

    def _compute_depreciation(self, cr, uid, asset, depreciation_counter): 

        amount = 0.0

        if asset.method == 'linear':
            #per month depreciation
            amount = (asset.purchase_value - asset.salvage_value) / (asset.method_number)
        else:
            book_value = asset.book_value
            asset_life_time_year = int(math.ceil(asset.method_number / 12))
            for x in range(1, asset_life_time_year+1):
                if x == asset_life_time_year:
                    amount = book_value / 12.0
                else:
                    amount = (book_value * asset.method_progress_factor) / 12.0

                y = (depreciation_counter / 12) + 1
                book_value -= (amount * 12.0)

                if x == y:
                    break

        return amount


    def _check_date(self, cr, uid, id):
        #First depreciation date has to be bigger or equal to acquisition date

        asset = self.browse(cr, uid, [id])[0]

        if asset.first_depreciation_date >= asset.purchase_date:
            return True
        else:
            return False

account_asset_asset()


class account_asset_depreciation_line(osv.osv):
    _name = 'account.asset.depreciation.line'
    _inherit = 'account.asset.depreciation.line'

    _columns =  {
                'remaining_value' : fields.float(string='To Be Depreciated', 
                    digits_compute=dp.get_precision('Account'),
                    required=True,
                    ),
                'depreciated_value' : fields.float(string='Accumulated Depreciation', 
                    digits_compute=dp.get_precision('Account'),
                    required=True,
                    ),
                }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
