# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv

class account_invoice_line(osv.osv):
    _name = 'account.invoice.line'
    _inherit = 'account.invoice.line'

    _columns =  {
                'direct_analytic_ids' : fields.one2many(string='Direct Analytics',
                    obj='account.account_invoice_line_analytic',
                    fields_id='account_invoice_line_id',
                    ),
                'indirect_analytic_ids' : fields.one2many(string='Indirect Analytics',
                    obj='account.account_invoice_line_analytic_distribution',
                    fields_id='account_invoice_line_id',
                    ),
                }

    def _get_detail_amount(self, cr, uid, line):
        obj_currency = self.pool.get('res.currency')
        invoice = line.invoice_id
        

        result =    {
                    'debit' : 0.0,
                    'credit' : 0.0,
                    'amount_currency' : 0.0,
                    'currency_id' : False,
                    }

        invoice_currency = invoice.currency_id
        company_currency = invoice.company_id.currency_id

        if invoice_currency.id != company_currency.id:
            result['currency_id'] = invoice_currency.id

        exchange_rate = obj_currency.compute(cr, uid, invoice_currency.id, company_currency.id, 1.0, context={'date' : invoice.date_invoice})
        amount_total = line.price_subtotal * exchange_rate

        if invoice.type == 'out_invoice' or invoice.type == 'in_refund':
            result['credit'] = amount_total
            result['amount_currency'] = result['currency_id'] and -1.0 * line.price_subtotal or 0.0
        else:
            result['debit'] = amount_total
            result['amount_currency'] = result['currency_id'] and 1.0 * line.price_subtotal or 0.0

        return result

    def _create_account_move_line(self, cr, uid, line, move_id):
        obj_line = self.pool.get('account.move.line')
        obj_period = self.pool.get('account.period')

        invoice = line.invoice_id

        period_ids = obj_period.find(cr, uid, invoice.date_invoice)
        if period_ids:
            period_id = period_ids[0]

        amount = self._get_detail_amount(cr, uid, line)

        value = {
                'move_id' : move_id,
                'name' : line.name,
                'date' : invoice.date_invoice,
                'date_maturity' : invoice.date_due,
                'period_id' : period_id,
                'debit' : amount['debit'],
                'credit' : amount['credit'],
                'account_id' : line.account_id.id,
                'partner_id' : invoice.partner_id.id,
                'currency_id' : amount['currency_id'],
                'amount_currency' : amount['amount_currency'],
                'product_id' : line.product_id and line.product_id.id or False,
                'quantity' : line.quantity,
                'product_uom_id' : line.uos_id and line.uos_id.id or False,
                'direct_analytic_ids' : [],
                'indirect_analytic_ids' : [],
                }

        # direct analytic
        if line.direct_analytic_ids:
            for direct in line.direct_analytic_ids:
                value_direct =  {
                                'cost_object_id' : direct.cost_object_id.id,
                                'analytic_account_id' : direct.analytic_account_id.id,
                                }

                value['direct_analytic_ids'].append((0,0,value_direct))

        # indirect analytic
        if line.indirect_analytic_ids:
            for indirect in line.indirect_analytic_ids:
                value_indirect =    {
                                    'cost_object_id' : indirect.cost_object_id.id,
                                    'analytic_distribution_id' : indirect.analytic_distribution_id.id,
                                    }

                value['indirect_analytic_ids'].append((0,0,value_indirect))

        move_id = obj_line.create(cr, uid, value)

        return move_id
account_invoice_line()




