# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


import openerp.addons.decimal_precision as dp
from osv import fields, osv
from openerp.tools.translate import _
from openerp import netsvc
import time

class account_move_line(osv.osv):
    _name = 'account.move.line'
    _inherit = 'account.move.line'

    def _amount_residual(self, cr, uid, ids, field_names, args, context=None):
        """
           This function returns the residual amount on a receivable or payable account.move.line.
           By default, it returns an amount in the currency of this journal entry (maybe different
           of the company currency), but if you pass 'residual_in_company_currency' = True in the
           context then the returned amount will be in company currency.
        """
        res = {}
        if context is None:
            context = {}
        cur_obj = self.pool.get('res.currency')
        for move_line in self.browse(cr, uid, ids, context=context):
            res[move_line.id] = {
                'amount_residual': 0.0,
                'amount_residual_currency': 0.0,
            }

            if move_line.reconcile_id:
                continue
            if not move_line.account_id.reconcile:
                #this function does not suport to be used on move lines not related to payable or receivable accounts
                continue

            if move_line.currency_id:
                move_line_total = move_line.amount_currency
                sign = move_line.amount_currency < 0 and -1 or 1
            else:
                move_line_total = move_line.debit - move_line.credit
                sign = (move_line.debit - move_line.credit) < 0 and -1 or 1
            line_total_in_company_currency =  move_line.debit - move_line.credit
            context_unreconciled = context.copy()
            if move_line.reconcile_partial_id:
                for payment_line in move_line.reconcile_partial_id.line_partial_ids:
                    if payment_line.id == move_line.id:
                        continue

                    if context_unreconciled.get('date_as_of', False) and payment_line.date > context_unreconciled['date_as_of']:
                        continue

                    if payment_line.currency_id and move_line.currency_id and payment_line.currency_id.id == move_line.currency_id.id:
                            move_line_total += payment_line.amount_currency
                    else:
                        if move_line.currency_id:
                            context_unreconciled.update({'date': payment_line.date})
                            amount_in_foreign_currency = cur_obj.compute(cr, uid, move_line.company_id.currency_id.id, move_line.currency_id.id, (payment_line.debit - payment_line.credit), round=False, context=context_unreconciled)
                            move_line_total += amount_in_foreign_currency
                        else:
                            move_line_total += (payment_line.debit - payment_line.credit)
                    line_total_in_company_currency += (payment_line.debit - payment_line.credit)

            result = move_line_total
            res[move_line.id]['amount_residual_currency'] =  sign * (move_line.currency_id and self.pool.get('res.currency').round(cr, uid, move_line.currency_id, result) or result)
            res[move_line.id]['amount_residual'] = sign * line_total_in_company_currency
        return res

    def _function_rate_computation(self, cr, uid, ids, field_name, args, context=None):
        res = {}

        for line in self.browse(cr, uid, ids):
            res[line.id] =   {
                            'rate_direct' : 0.0,
                            'rate_indirect' : 0.0,
                            }

            if line.currency_id:
                try:
                    res[line.id]['rate_indirect'] = abs(line.amount_currency) / (line.debit or line.credit or 0.0)
                except:
                    res[line.id]['rate_indirect'] = 0.0

                try:
                    res[line.id]['rate_direct'] = (line.debit or line.credit or 0.0) / abs(line.amount_currency)
                except:
                    res[line.id]['rate_direct'] = 0.0

        return res
                

    _columns =  {
                'name' : fields.char(string='Name', size=255, required=True),
                'amount_residual_currency': fields.function(_amount_residual, string='Residual Amount in Currency', multi="residual", help="The residual amount on a receivable or payable of a journal entry expressed in its currency (maybe different of the company currency)."),
                'amount_residual': fields.function(_amount_residual, string='Residual Amount', multi="residual", help="The residual amount on a receivable or payable of a journal entry expressed in the company currency."),

                'rate_indirect' : fields.function(string='Rate Indirect', fnct=_function_rate_computation, type='float', digits=(12,12), method=True, store=False, multi='rate_computation'),
                'rate_direct' : fields.function(string='Rate Direct', fnct=_function_rate_computation, type='float', digits_compute=dp.get_precision('account'), method=True, store=False, multi='rate_computation'),

                'direct_analytic_ids' : fields.one2many(string='Direct Analytics',
                    obj='account.account_move_line_analytic',
                    fields_id='account_move_line_id',
                    ),
                'indirect_analytic_ids' : fields.one2many(string='Indirect Analytics',
                    obj='account.account_move_line_analytic_distribution',
                    fields_id='account_move_line_id',
                    ),
                }

    def reconcile(self, cr, uid, ids, type='auto', writeoff_acc_id=False, writeoff_period_id=False, writeoff_journal_id=False, context=None):
        account_obj = self.pool.get('account.account')
        move_obj = self.pool.get('account.move')
        move_rec_obj = self.pool.get('account.move.reconcile')
        partner_obj = self.pool.get('res.partner')
        currency_obj = self.pool.get('res.currency')
        lines = self.browse(cr, uid, ids, context=context)
        unrec_lines = filter(lambda x: not x['reconcile_id'], lines)
        credit = debit = 0.0
        currency = 0.0
        account_id = False
        partner_id = False
        if context is None:
            context = {}
        company_list = []
        for line in self.browse(cr, uid, ids, context=context):
            if company_list and not line.company_id.id in company_list:
                raise osv.except_osv(_('Warning!'), _('To reconcile the entries company should be the same for all entries.'))
            company_list.append(line.company_id.id)
        for line in unrec_lines:
            if line.state <> 'valid':
                raise osv.except_osv(_('Error!'),
                        _('Entry "%s" is not valid !') % line.name)
            credit += line['credit']
            debit += line['debit']
            currency += line['amount_currency'] or 0.0
            account_id = line['account_id']['id']
            partner_id = (line['partner_id'] and line['partner_id']['id']) or False
        writeoff = debit - credit

        # Ifdate_p in context => take this date
        if context.has_key('date_p') and context['date_p']:
            date=context['date_p']
        else:
            date = time.strftime('%Y-%m-%d')

        cr.execute('SELECT account_id, reconcile_id '\
                   'FROM account_move_line '\
                   'WHERE id IN %s '\
                   'GROUP BY account_id, reconcile_id',
                   (tuple(ids), ))
        r = cr.fetchall()
        #TODO: move this check to a constraint in the account_move_reconcile object
        # if len(r) != 1:
        #     raise osv.except_osv(_('Error'), _('Entries are not of the same account or already reconciled ! '))
        if not unrec_lines:
            raise osv.except_osv(_('Error!'), _('Entry is already reconciled.'))
        account = account_obj.browse(cr, uid, account_id, context=context)
        if not account.reconcile:
            raise osv.except_osv(_('Error'), _('The account is not defined to be reconciled !'))
        if r[0][1] != None:
            raise osv.except_osv(_('Error!'), _('Some entries are already reconciled.'))

        if (not currency_obj.is_zero(cr, uid, account.company_id.currency_id, writeoff)) or \
           (account.currency_id and (not currency_obj.is_zero(cr, uid, account.currency_id, currency))):
            if not writeoff_acc_id:
                raise osv.except_osv(_('Warning!'), _('You have to provide an account for the write off/exchange difference entry.'))
            if writeoff > 0:
                debit = writeoff
                credit = 0.0
                self_credit = writeoff
                self_debit = 0.0
            else:
                debit = 0.0
                credit = -writeoff
                self_credit = 0.0
                self_debit = -writeoff
            # If comment exist in context, take it
            if 'comment' in context and context['comment']:
                libelle = context['comment']
            else:
                libelle = _('Write-Off')

            cur_obj = self.pool.get('res.currency')
            cur_id = False
            amount_currency_writeoff = 0.0
            if context.get('company_currency_id',False) != context.get('currency_id',False):
                cur_id = context.get('currency_id',False)
                for line in unrec_lines:
                    if line.currency_id and line.currency_id.id == context.get('currency_id',False):
                        amount_currency_writeoff += line.amount_currency
                    else:
                        tmp_amount = cur_obj.compute(cr, uid, line.account_id.company_id.currency_id.id, context.get('currency_id',False), abs(line.debit-line.credit), context={'date': line.date})
                        amount_currency_writeoff += (line.debit > 0) and tmp_amount or -tmp_amount

            writeoff_lines = [
                (0, 0, {
                    'name': libelle,
                    'debit': self_debit,
                    'credit': self_credit,
                    'account_id': account_id,
                    'date': date,
                    'partner_id': partner_id,
                    'currency_id': cur_id or (account.currency_id.id or False),
                    'amount_currency': amount_currency_writeoff and -1 * amount_currency_writeoff or (account.currency_id.id and -1 * currency or 0.0)
                }),
                (0, 0, {
                    'name': libelle,
                    'debit': debit,
                    'credit': credit,
                    'account_id': writeoff_acc_id,
                    'analytic_account_id': context.get('analytic_id', False),
                    'date': date,
                    'partner_id': partner_id,
                    'currency_id': cur_id or (account.currency_id.id or False),
                    'amount_currency': amount_currency_writeoff and amount_currency_writeoff or (account.currency_id.id and currency or 0.0)
                })
            ]

            writeoff_move_id = move_obj.create(cr, uid, {
                'period_id': writeoff_period_id,
                'journal_id': writeoff_journal_id,
                'date':date,
                'state': 'draft',
                'line_id': writeoff_lines
            })

            writeoff_line_ids = self.search(cr, uid, [('move_id', '=', writeoff_move_id), ('account_id', '=', account_id)])
            if account_id == writeoff_acc_id:
                writeoff_line_ids = [writeoff_line_ids[1]]
            ids += writeoff_line_ids

        r_id = move_rec_obj.create(cr, uid, {
            'type': type,
            'line_id': map(lambda x: (4, x, False), ids),
            'line_partial_ids': map(lambda x: (3, x, False), ids)
        })
        wf_service = netsvc.LocalService("workflow")
        # the id of the move.reconcile is written in the move.line (self) by the create method above
        # because of the way the line_id are defined: (4, x, False)
        for id in ids:
            wf_service.trg_trigger(uid, 'account.move.line', id, cr)

        if lines and lines[0]:
            partner_id = lines[0].partner_id and lines[0].partner_id.id or False
            if partner_id and not partner_obj.has_something_to_reconcile(cr, uid, partner_id, context=context):
                partner_obj.mark_as_reconciled(cr, uid, [partner_id], context=context)
        return r_id

    def create_analytic_lines(self, cr, uid, ids, context=None):
        if context is None:
            context = {}

        analytic_line_obj = self.pool.get('account.analytic.line')

        for line in self.browse(cr, uid, ids, context=context):
            if line.analytics_id:
                if not line.journal_id.analytic_journal_id:
                    raise osv.except_osv(_('No Analytic Journal!'),_("You have to define an analytic journal on the '%s' journal.") % (line.journal_id.name,))

                # toremove = analytic_line_obj.search(cr, uid, [('move_id','=',line.id),('new_analytic','=',False)], context=context)
                toremove = analytic_line_obj.search(cr, uid, [('move_id','=',line.id)], context=context)

                if toremove:
                    analytic_line_obj.unlink(cr, uid, toremove, context=context)

                for line2 in line.analytics_id.account_ids:
                    val = (line.credit or  0.0) - (line.debit or 0.0)
                    amt=val * (line2.rate/100)
                    al_vals={
                        'name': line.name,
                        'date': line.date,
                        'account_id': line2.analytic_account_id.id,
                        'unit_amount': line.quantity,
                        'product_id': line.product_id and line.product_id.id or False,
                        'product_uom_id': line.product_uom_id and line.product_uom_id.id or False,
                        'amount': amt,
                        'general_account_id': line.account_id.id,
                        'move_id': line.id,
                        'journal_id': line.journal_id.analytic_journal_id.id,
                        'ref': line.ref,
                        'percentage': line2.rate
                    }

                analytic_line_obj.create(cr, uid, al_vals, context=context)

            if not self._create_analytic_accounting_data(cr, uid, line):
                return False

        return True

    def _create_analytic_accounting_data(self, cr, uid, move):
        obj_analytic_line = self.pool.get('account.analytic.line')

        criteria = [('move_id','=',move.id),('new_analytic','=',True)]
        line_ids = obj_analytic_line.search(cr, uid, criteria)
        obj_analytic_line.unlink(cr, uid, line_ids)

        if not self._create_direct_analytic_data(cr, uid, move):
            return False


        if not self._create_indirect_analytic_data(cr, uid, move):
            return False

        return True

    def _create_direct_analytic_data(self, cr, uid, move):
        obj_analytic_line = self.pool.get('account.analytic.line')

        if move.direct_analytic_ids:
            for direct in move.direct_analytic_ids:

                amount = (move.credit or  0.0) - (move.debit or 0.0)

                value =     {
                            'name': move.name,
                            'date': move.date,
                            'account_id': direct.analytic_account_id.id,
                            'unit_amount': move.quantity,
                            'product_id': move.product_id and move.product_id.id or False,
                            'product_uom_id': move.product_uom_id and move.product_uom_id.id or False,
                            'amount': amount,
                            'general_account_id': move.account_id.id,
                            'move_id': move.id,
                            'journal_id': move.journal_id.analytic_journal_id.id,
                            'ref': move.ref,
                            'percentage': 100.00,
                            'new_analytic' : True,
                            }

                obj_analytic_line.create(cr, uid, value)

        return True

    def _create_indirect_analytic_data(self, cr, uid, move):
        obj_analytic_line = self.pool.get('account.analytic.line')

        if move.indirect_analytic_ids:
            for indirect in move.indirect_analytic_ids:
                if indirect.analytic_distribution_id.account_ids:
                    for plan in indirect.analytic_distribution_id.account_ids:
                        amount = ((move.credit or  0.0) - (move.debit or 0.0)) * (plan.rate / 100.0)
                        value =     {
                                    'name': move.name,
                                    'date': move.date,
                                    'account_id': plan.analytic_account_id.id,
                                    'unit_amount': move.quantity,
                                    'product_id': move.product_id and move.product_id.id or False,
                                    'product_uom_id': move.product_uom_id and move.product_uom_id.id or False,
                                    'amount': amount,
                                    'general_account_id': move.account_id.id,
                                    'move_id': move.id,
                                    'journal_id': move.journal_id.analytic_journal_id.id,
                                    'ref': move.ref,
                                    'percentage': plan.rate,
                                    'new_analytic' : True,
                                    }

                        obj_analytic_line.create(cr, uid, value)

        return True
account_move_line()




