# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv
import decimal_precision as dp
from openerp.tools.translate import _

class account_voucher_line(osv.osv):
    _name = 'account.voucher.line'
    _inherit = 'account.voucher.line'

    def _default_product_qty(self, cr, uid, context={}):
        return 1.0

    def function_amount_all(self, cr, uid, ids, name, args, context={}):
        res = {}
        for line in self.browse(cr, uid, ids):
            res[line.id] =  {
                            'total_dr' : 0.0,
                            'total_cr' : 0.0,
                            }
            if line.type == 'dr':
                res[line.id]['total_dr'] = line.amount_in_company_currency + line.amount_tax_in_company_currency
            else:
                res[line.id]['total_cr'] = line.amount_in_company_currency + line.amount_tax_in_company_currency
        return res

    def _compute_balance(self, cr, uid, ids, name, args, context=None):
        obj_currency = self.pool.get('res.currency')
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            res[line.id] =  {
                            'amount_tax' : 0.0,
                            'amount_after_tax' : 0.0,
                            'amount_tax_in_company_currency' : 0.0,
                            'amount_original' : 0.0, #TODO:OBSOLETE
                            'amount_unreconciled' : 0.0, #TODO:OBSOLETE
                            'amount_in_company_currency' : 0.0,
                            'amount_currency' : 0.0,
                            'amount_currency_diff' : 0.0,
                            'amount_company_currency_move_date' : 0.0,
                            'amount_move_currency_voucher_date' : 0.0,
                            'amount_diff_in_company_currency' : 0.0,
                            'function_exchange_rate' : 0.0,
                            'function_rate_conversion_method' : 'indirect',
                            }


            ctx = context.copy()
            ctx.update({'date': line.voucher_id.date})

            move_line = line.move_line_id or False

            company_currency_id = line.voucher_id.company_id.currency_id.id
            voucher_currency_id = line.voucher_id.journal_id.currency and line.voucher_id.journal_id.currency.id or company_currency_id
            voucher_rate = line.voucher_id.exchange_rate

            if move_line:
                move_rate = obj_currency.compute(cr, uid, voucher_currency_id, company_currency_id, 1.0, context={'date' : move_line.date})

            # DETERMINED RATE CONVERSION METHOD
            # IF RATE OPTIONS == SYSTEM THEN IT WOULD USE COMPANY RATE CONVERSION METHOD
            # ELSE THEN IT WOULD USE THE OPTIONS
            if line.rate_options == 'system':
                res[line.id]['function_rate_conversion_method'] = line.voucher_id.company_id.exchange_rate_method
                if move_line:
                    res[line.id]['function_exchange_rate'] = move_rate

                else:
                    res[line.id]['function_exchange_rate'] = voucher_rate
            else:
                res[line.id]['function_rate_conversion_method'] = line.rate_conversion_method
                res[line.id]['function_exchange_rate'] = line.exchange_rate

            #TAX
            for tax in line.tax_line_ids:
                res[line.id]['amount_tax_in_company_currency'] += tax.function_tax_amount_in_company_currency

            if res[line.id]['function_rate_conversion_method'] == 'direct':
                res[line.id]['amount_in_company_currency'] = (line.amount * res[line.id]['function_exchange_rate'])
                res[line.id]['amount_tax'] = (res[line.id]['amount_tax_in_company_currency'] / res[line.id]['function_exchange_rate'])
                amount_in_company_currency_at_voucher_date = (line.amount * voucher_rate)
            else:
                res[line.id]['amount_in_company_currency']  = line.amount * (1.0 / res[line.id]['function_exchange_rate'])
                res[line.id]['amount_tax'] = res[line.id]['amount_tax_in_company_currency'] * res[line.id]['function_exchange_rate']
                amount_in_company_currency_at_voucher_date = line.amount * (1.0 / voucher_rate)


            res[line.id]['amount_diff_in_company_currency'] = amount_in_company_currency_at_voucher_date - res[line.id]['amount_in_company_currency']
            res[line.id]['amount_after_tax'] = line.amount + res[line.id]['amount_tax']

        return res

    def _default_rate_options(self, cr, uid, context={}):
        return 'system'

    def _default_rate_conversion_method(self, cr, uid, context={}):
        return 'indirect'

    def _default_exchange_rate(self, cr, uid, context={}):
        return 1.0

    _columns =   {
                'account_id' : fields.many2one(string='Account', obj='account.account', required=True),
                'total_dr' : fields.function(string='Total Debit', fnct=function_amount_all, type='float', digits_compute=dp.get_precision('Account'), method=True, store=True, multi='all'),
                'total_cr' : fields.function(string='Total Credit', fnct=function_amount_all, type='float', digits_compute=dp.get_precision('Account'), method=True, store=True, multi='all'),

                'product_id' : fields.many2one(obj='product.product', string='Product', readonly=False, ondelete='restrict'),
                'product_uom_id' : fields.many2one(string='UoM', obj='product.uom', ondelete='restrict'),
                'product_qty' : fields.float(string='Qty'),
                'product_unit_price' : fields.float(string='Price'),

                'partner_id' : fields.many2one(string='Partner', obj='res.partner', ondelete='restrict'),
                'amount_original': fields.function(_compute_balance, multi='dc', type='float', string='Original Amount', store=True, digits_compute=dp.get_precision('Account')),
                'amount_unreconciled': fields.function(_compute_balance, multi='dc', type='float', string='Open Balance', store=True, digits_compute=dp.get_precision('Account')),
                'analytic_plan_id' : fields.many2one(string='Analytic Distribution', obj='account.analytic.plan.instance'),

                'amount_in_company_currency' : fields.function(fnct=_compute_balance, multi='dc', type='float', string='Amount In Company Currency', store=False, digits_compute=dp.get_precision('Account')),
                'amount_currency' : fields.function(fnct=_compute_balance, multi='dc', type='float', string='Amount In Move Currency At Move Date', store=False, digits_compute=dp.get_precision('Account')),
                'amount_currency_diff' : fields.function(fnct=_compute_balance, multi='dc', type='float', string='Diff Amount In Move Currency', store=False, digits_compute=dp.get_precision('Account')),
                'amount_move_currency_voucher_date' : fields.function(fnct=_compute_balance, multi='dc', type='float', string='Amount In Move Currency At Voucher Date', store=False, digits_compute=dp.get_precision('Account')),
                'amount_diff_in_company_currency' : fields.function(fnct=_compute_balance, multi='dc', type='float', string='Diff Amount In Company Currency', store=False, digits_compute=dp.get_precision('Account')),
                'amount_company_currency_move_date' : fields.function(fnct=_compute_balance, multi='dc', type='float', string='Amount In Company Currency At Move Date', store=False, digits_compute=dp.get_precision('Account')),

                'amount_tax' : fields.function(fnct=_compute_balance, multi='dc', type='float', 
                    string='Amount Tax', store=False, digits_compute=dp.get_precision('Account')),
                'amount_after_tax' : fields.function(fnct=_compute_balance, multi='dc', type='float', 
                    string='Amount After Tax', store=False, digits_compute=dp.get_precision('Account')),
                'amount_tax_in_company_currency' : fields.function(fnct=_compute_balance, multi='dc', type='float', 
                    string='Amount Tax In Company Currency', store=False, digits_compute=dp.get_precision('Account')),

                'rate_options' : fields.selection(string='Exchange Rate Options', selection=[('manual','Manual'),('system','System')], required=True),
                'rate_conversion_method' : fields.selection(string='Rate Convertion Method', selection=[('direct','Direct'),('indirect','Indirect')], required=True),
                'function_rate_conversion_method' : fields.function(string='Rate Conversion Method', fnct=_compute_balance, selection=[('direct','Direct'),('indirect','Indirect')], type='selection', method=True, store=False, multi='dc'),
                'function_exchange_rate' : fields.function(fnct=_compute_balance, multi='dc', type='float', string='Exchange Rate', method=True, store=False, digits_compute=dp.get_precision('Account')),
                'exchange_rate' : fields.float(string='Exchange Rate', digits_compute=dp.get_precision('Account'), required=True),

                'tax_line_ids' : fields.one2many(string='Tax', obj='account.voucher_line_tax',
                    fields_id='voucher_line_id'),

                'direct_analytic_ids' : fields.one2many(string='Direct Analytics',
                    obj='account.account_voucher_line_analytic',
                    fields_id='account_voucher_line_id',
                    ),
                'indirect_analytic_ids' : fields.one2many(string='Indirect Analytics',
                    obj='account.account_voucher_line_analytic_distribution',
                    fields_id='account_voucher_line_id',
                    ),

                }

    _defaults = {
                'rate_options' : _default_rate_options,
                'rate_conversion_method' : _default_rate_conversion_method,
                'exchange_rate' : _default_exchange_rate,
                'product_qty' : _default_product_qty,
                }

    def onchange_price_qty(self, cr, uid, ids, unit_price=1.0, qty=1.0, product_id=False, uom_id=False):
        value = {}
        domain = {}
        warning = {}

        obj_product = self.pool.get('product.product')
        obj_uom = self.pool.get('product.uom')

        value.update({'amount' : unit_price * qty})

        if product_id and uom_id:
            product = obj_product.browse(cr, uid, [product_id])[0]
            to_uom_id = product.uom_id.id
            from_uom_id = uom_id
            new_qty = obj_uom._compute_qty(cr, uid, from_uom_id, qty, to_uom_id)
            value['amount'] = unit_price * new_qty

        return {'value' : value, 'domain' : domain, 'warning' : warning}
                
    def onchange_product_id(self, cr, uid, ids, line_type, product_id=False, qty=1.0, uom_id=False, unit_price=1.0):

        obj_res_product = self.pool.get('product.product')

        value = {
                'amount' : unit_price * qty,
                'product_unit_price' : unit_price,
                }
        domain = {}
        warning = {}


        if product_id:
            value.update({'name' : '', 'account_id' : False, 'product_uom_id' : False})
            product = obj_res_product.browse(cr, uid, [product_id])[0]
            if line_type:
                if line_type == 'dr':
                    value['product_unit_price'] = product.standard_price
                    value['product_uom_id'] = product.uom_id and product.uom_id.id
                    if product.property_account_expense:
                        value['account_id'] = product.property_account_expense.id
                    else:
                        if product.categ_id.property_account_expense_categ:
                            value['account_id'] = product.categ_id.property_account_expense_categ.id
                        else:
                            value['account_id'] = False
                elif line_type == 'cr':
                    value['product_unit_price'] = product.list_price
                    value['uom_id'] = product.uom_po_id and product.uom_po_id.id
                    if product.property_account_income:
                        value['account_id'] = product.property_account_income.id
                    else:
                        if product.categ_id.property_account_income_categ:
                            value['account_id'] = product.categ_id.property_account_income_categ.id
                        else:
                            value['account_id'] = False

            price = self.onchange_price_qty(cr, uid, ids, value['product_unit_price'], qty, product_id, uom_id)
            value['amount'] = price['value']['amount']
            value['name'] = obj_res_product.name_get(cr, uid, [product_id])[0][1]



            domain.update({
                        'product_uom_id' : [('category_id','=',product.uom_id.category_id.id)],
                        })

        
        return {'value' : value, 'domain' : domain, 'warning' : warning}

    

    def default_get(self, cr, uid, fields_list, context=None):
        res = super(account_voucher_line, self).default_get(cr, uid, fields_list, context)

        if context.get('default_detail_type_selection', False):
            res.update({'type' : context.get('default_detail_type_selection', False)})
    
        return res

    def compute_amount(self, cr, uid, voucher_id, move_id, journal_id, currency_id):
        currency_pool = self.pool.get('res.currency')
        obj_move = self.pool.get('account.move.line')
        obj_journal = self.pool.get('account.journal')
        obj_voucher = self.pool.get('account.voucher')
        
        rs = {}
        
        line = obj_move.browse(cr, uid, [move_id])[0]
        journal = obj_journal.browse(cr, uid, [journal_id])[0]
        voucher = obj_voucher.browse(cr, uid, [voucher_id])[0]
        
        currency_id = currency_id or journal.company_id.currency_id.id
        company_currency = journal.company_id.currency_id.id

        source_currency_id = line.currency_id and line.currency_id.id or journal.company_id.currency_id.id
        destination_currency_id = voucher.journal_id.currency and voucher.journal_id.currency.id or journal.company_id.currency_id.id


        context = {'date' : voucher.date}
        
        if line.currency_id == currency_id:
            amount_original = abs(line.amount_currency)
            amount_unreconciled = abs(line.amount_residual_currency)
        else:
            rate = currency_pool.compute(cr, uid, source_currency_id, destination_currency_id, 1.0, context=context)

            amount_original = rate * (line.credit or line.debit or 0.0)
            amount_unreconciled = rate * abs(line.amount_residual_currency)

        rs =    {
                'amount_original': amount_original,
                'amount': amount_unreconciled,
                'amount_unreconciled': amount_unreconciled,
                }

        return rs
        
    def onchange_move_id(self, cr, uid, ids, move_id, journal_id, currency_id):
        value = {}
        domain = {}
        warning = {}
        
        obj_move = self.pool.get('account.move.line')
                
        if move_id:
            move = obj_move.browse(cr, uid, [move_id])[0]
            value['account_id'] = move.account_id.id
            value['name'] = move.name
            value['date_original'] = move.date
            value['date_due'] = move.date_maturity
            res = self.compute_amount(cr, uid, move_id, journal_id, currency_id)
            value['amount'] = res['amount']
            value['amount_original'] = res['amount_original']
            value['amount_unreconciled'] = res['amount_unreconciled']
            value['partner_id'] = move.partner_id and move.partner_id.id or False
            
        return {'value' : value, 'domain' : domain, 'warning' : warning}        

    def onchange_partner_id(self, cr, uid, ids, move_id):
        value = {}
        domain = {}
        warning = {}

        obj_move = self.pool.get('account.move.line')

        if move_id:
            move = obj_move.browse(cr, uid, [move_id])[0]
            value['partner_id'] = move.partner_id and move.partner_id.id or False

        return {'value' : value, 'domain' : domain, 'warning' : warning}

    def _get_detail_account(self, cr, uid, id):
        detail = self.browse(cr, uid, [id])[0]
        voucher_type = detail.voucher_id.voucher_type_id
        journal = detail.voucher_id.journal_id

        if voucher_type.detail_account == 'voucher_detail_account':
            return detail.account_id.id
        elif voucher_type.detail_account == 'journal_temporary_income_account':
            return journal.temporary_income_account_id.id
        elif voucher_type.detail_account == 'journal_temporary_expense_account':
            return journal.temporary_expense_account_id.id
        else:
            return False


    def _move_item_debit_credit_amount(self, cr, uid, type='dr', amount=0.0, currency_id=False, amount_currency=0.0):
        result =    {
                    'debit' : 0.0,
                    'credit' : 0.0,
                    'currency_id' : currency_id,
                    'amount_currency' : 0.0,
                    }

        if type == 'dr':
            if amount > 0:
                result['debit'] = abs(amount)
                result['amount_currency'] = currency_id and abs(amount_currency) or 0.0
            elif amount < 0:
                result['credit'] = abs(amount)
                result['amount_currency'] = currency_id and -1.0 * abs(amount_currency) or 0.0
        elif type == 'cr':
            if amount > 0:
                result['credit'] = abs(amount)
                result['amount_currency'] = currency_id and -1.0 * abs(amount_currency) or 0.0
            elif amount < 0:
                result['debit'] = abs(amount)
                result['amount_currency'] = currency_id and abs(amount_currency) or 0.0
        return result

    def _prepare_detail_move_line(self, cr, uid, id, move_id):
        detail = self.browse(cr, uid, [id])[0]
        voucher = detail.voucher_id

        debit_credit_amount = self._move_item_debit_credit_amount(cr, uid, detail.type, detail.amount_in_company_currency, voucher.journal_id.currency and voucher.journal_id.currency.id or False, detail.amount)

        result =    {
                    'name' : detail.name,
                    'debit' : debit_credit_amount['debit'],
                    'credit' : debit_credit_amount['credit'],
                    'account_id' : self._get_detail_account(cr, uid, id),
                    'move_id' : move_id,
                    'amount_currency' : debit_credit_amount['amount_currency'],
                    'currency_id' : debit_credit_amount['currency_id'],
                    'product_id' : detail.product_id and detail.product_id.id or False,
                    'quantity' : detail.product_qty,
                    'product_uom_id' : detail.product_uom_id and detail.product_uom_id.id or False,
                    'partner_id' : detail.partner_id and detail.partner_id.id or False,
                    'analytics_id' : detail.analytic_plan_id and detail.analytic_plan_id.id or False, 
                    'direct_analytic_ids' : [],
                    'indirect_analytic_ids' : []
                    }

        if detail.direct_analytic_ids:
            for direct in detail.direct_analytic_ids:
                value_direct =  {
                                'cost_object_id' : direct.cost_object_id.id,
                                'analytic_account_id' : direct.analytic_account_id.id,
                                }
                result['direct_analytic_ids'].append((0,0,value_direct))

        if detail.indirect_analytic_ids:
            for indirect in detail.indirect_analytic_ids:
                value_indirect =    {
                                    'cost_object_id' : indirect.cost_object_id.id,
                                    'analytic_distribution_id' : indirect.analytic_distribution_id.id,
                                    }   
                result['indirect_analytic_ids'].append((0,0,value_indirect))

        return result

    def _prepare_detail_move_line_exchange(self, cr, uid, id, move_id):
        detail = self.browse(cr, uid, [id])[0]
        voucher = detail.voucher_id

        if detail.type == 'dr':
            if detail.amount_diff_in_company_currency < 0:
                move_type = 'cr'
            else:
                move_type = 'dr'
        else:
            if detail.amount_diff_in_company_currency < 0:
                move_type = 'dr'
            else:
                move_type = 'cr'


        if move_type == 'dr':
            account = detail.voucher_id.company_id.expense_currency_exchange_account_id
            if not account:
                raise osv.except_osv(_('Warning!'),_('No exchange loss account defined'))
        else:
            account = detail.voucher_id.company_id.income_currency_exchange_account_id
            if not account:
                raise osv.except_osv(_('Warning!'),_('No exchange profit account defined'))

        debit_credit_amount = self._move_item_debit_credit_amount(cr, uid, move_type, detail.amount_diff_in_company_currency, False, 0.0)

        result =    {
                    'name' : _('Currency rate adjustment : %s' % (detail.name)),
                    'debit' : debit_credit_amount['debit'],
                    'credit' : debit_credit_amount['credit'],
                    'account_id' : account.id,
                    'move_id' : move_id,
                    'currency_id' : False,
                    'amount_currency' : 0.0,
                    'direct_analytic_ids' : [],
                    'indirect_analytic_ids' : [],
                    }

        if detail.direct_analytic_ids:
            for direct in detail.direct_analytic_ids:
                value_direct =  {
                                'cost_object_id' : direct.cost_object_id.id,
                                'analytic_account_id' : direct.analytic_account_id.id,
                                }
                result['direct_analytic_ids'].append((0,0,value_direct))

        if detail.indirect_analytic_ids:
            for indirect in detail.indirect_analytic_ids:
                value_indirect =    {
                                    'cost_object_id' : indirect.cost_object_id.id,
                                    'analytic_distribution_id' : indirect.analytic_distribution_id.id,
                                    }   
                result['indirect_analytic_ids'].append((0,0,value_indirect))

        return result






                            
account_voucher_line()                          
    
