# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution   
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from osv import fields, osv
from openerp.tools.translate import _
from openerp import netsvc
import openerp.addons.decimal_precision as dp

class cash_statement(osv.osv):
    _name = 'account.cash.statement'
    _inherit = ['account.bank.statement','mail.thread']
    _table = 'account_bank_statement'
    _description = 'Cash Opname'

    def _end_balance(self, cursor, user, ids, name, attr, context=None):
        res = {}
        for statement in self.browse(cursor, user, ids, context=context):
            res[statement.id] = statement.balance_start
            for line in statement.line_ids:
                res[statement.id] += line.amount

            for move_line in statement.move_line_ids:
                res[statement.id] += move_line.debit - move_line.credit
        return res

    def _get_statement(self, cr, uid, ids, context=None):
        result = {}
        for line in self.pool.get('account.cash.statement.line').browse(cr, uid, ids, context=context):
            result[line.statement_id.id] = True
        return result.keys()

    def default_state(self, cr, uid, context={}):
        return 'draft'

    def function_compute_balance(self, cr, uid, ids, fields_name, args, context=None):
        res = {}
        obj_fiscalyear = self.pool.get('account.fiscalyear')
        obj_account = self.pool.get('account.account')
        

        for cash_opname in self.browse(cr, uid, ids):
            fiscalyear_id = obj_fiscalyear.find(cr, uid, cash_opname.date)
            fiscalyear = obj_fiscalyear.browse(cr, uid, [fiscalyear_id])[0]
            date_start = fiscalyear.date_start
            date_stop_beginning = cash_opname.date
            date_stop_end = cash_opname.date_to

            account_id = cash_opname.journal_id.default_debit_account_id.id
            account_start = obj_account.browse(cr, uid, [account_id], context={'date_from' : date_start, 'date_to' : date_stop_beginning, 'state' : 'posted'})[0]
            account_end = obj_account.browse(cr, uid, [account_id], context={'date_from' : date_start, 'date_to' : date_stop_end, 'state' : 'posted'})[0]

            res[cash_opname.id] =  {

                                            'balance_end' : account_end.balance,
                                            'balance_start_compute' : account_start.balance,
                                            }

        return res

    def function_compute_cashbox_balance(self, cr, uid, ids, fields_name, args, context={}):
        res = {}

        for cash_opname in self.browse(cr, uid, ids):
            res[cash_opname.id] =   {
                                    'cash_balance_start' : 0.0,
                                    'cash_balance_end' : 0.0,
                                    }

            if cash_opname.detail_ids:
                for detail in cash_opname.detail_ids:
                    res[cash_opname.id]['cash_balance_start'] += detail.subtotal_opening
                    res[cash_opname.id]['cash_balance_end'] += detail.subtotal_closing

        return res



    _columns =  {
                'name' : fields.char(string='Reference', size=64, required=True, readonly=True),
                'date' : fields.date(string='Date From', 
                                        required=True,
                                        readonly=True,
                                        states={'draft':[('readonly',False)]}),
                'balance_end' : fields.function(string='Computed Ending Balance', fnct=function_compute_balance, type='float', method=True, store=False, multi='compute_balance'),
                'balance_start_compute' : fields.function(string='Computed Beginning Balance', fnct=function_compute_balance, type='float', method=True, store=False, multi='compute_balance'),
                'date_to' : fields.date(string='Date To', required=True, readonly=True, states={'draft':[('readonly',False)]}),
                'state' : fields.selection(string='State', selection=[('draft','Draft'),('confirmed','Waiting For Approval'),('approved','Ready To Process'),('done','Done'),('cancelled','Cancelled')], required=True, readonly=True),
                'move_line_ids' : fields.one2many(string='Journal Items', obj='account.move.line', fields_id='statement_id', readonly=True, states={'draft':[('readonly',False)]}),
                'balance_end_real' : fields.float(string='Real Ending Balance', digits_compute=dp.get_precision('Account'), readonly=True, states={'draft':[('readonly',False)]}),
                'balance_start' : fields.float(string='Real Starting Balance', digits_compute=dp.get_precision('Account'), readonly=True, states={'draft':[('readonly',False)]}),

                'detail_ids' : fields.one2many(string='Cashbox Lines', obj='account.cashbox.line', fields_id='bank_statement_id', readonly=True,states={'draft':[('readonly',False)]}),
                'cash_balance_start' : fields.function(string='Cash Starting Balance', fnct=function_compute_cashbox_balance, type='float', method=True, store=False, multi='cash_balance'),
                'cash_balance_end' : fields.function(string='Cash Ending Balance', fnct=function_compute_cashbox_balance, type='float', method=True, store=False, multi='cash_balance'),

                }

    _defaults = {
                'state' : default_state,
                }

    def _check_duration(self, cr, uid, ids, context=None):
        cash_opname = self.browse(cr, uid, ids)[0]

        if cash_opname.date > cash_opname.date_to:
            return False

        return True

    def _check_overlapping(self, cr, uid, ids, context=None):
        for cash_opname in self.browse(cr, uid, ids):
            result_ids = self.search(cr, uid, [('date','<',cash_opname.date),('date','<',cash_opname.date_to),('date_to','>',cash_opname.date),('date_to','<',cash_opname.date_to),('id','!=',cash_opname.id),('journal_id','=',cash_opname.journal_id.id)])
            result_ids += self.search(cr, uid, [('date','>',cash_opname.date),('date','<',cash_opname.date_to),('date_to','>',cash_opname.date),('date_to','>',cash_opname.date_to),('id','!=',cash_opname.id),('journal_id','=',cash_opname.journal_id.id)])
            result_ids += self.search(cr, uid, [('date','>',cash_opname.date),('date','<',cash_opname.date_to),('date_to','>',cash_opname.date),('date_to','<',cash_opname.date_to),('id','!=',cash_opname.id),('journal_id','=',cash_opname.journal_id.id)])
            result_ids += self.search(cr, uid, [('date','<',cash_opname.date),('date','<',cash_opname.date_to),('date_to','>',cash_opname.date),('date_to','>',cash_opname.date_to),('id','!=',cash_opname.id),('journal_id','=',cash_opname.journal_id.id)])
            result_ids += self.search(cr, uid, [('date','=',cash_opname.date),('date_to','=',cash_opname.date_to),('id','!=',cash_opname.id),('journal_id','=',cash_opname.journal_id.id)])


            if result_ids:
               return False

        return True


            

    _constraints =  [
                    (_check_duration, 'Error!\nDate From bigger than Date To',['date','date_to']),
                    (_check_overlapping, 'Error!\nOverlapping cash opname',['date','date_to']),
                    ]

    def copy(self, cr, uid, id, defaults, context=None):
        raise osv.except_osv(_('Warning!'),_('You can not copy data'))

        return False

    def unlink(self, cr, uid, ids, context=None):
        for cash_opname in self.browse(cr, uid, ids):
            if cash_opname.state != 'draft':
                raise osv.except_osv(_('Warning!'),_('You can not delete data unless it has Draft state'))
                return False

        return super(cash_statement, self).unlink(cr, uid, ids, context)

    def workflow_action_confirm(self, cr, uid, ids, context={}):
        for id in ids:
            if not self.check_balance(cr, uid, id):
                return False

            if not self.create_sequence(cr, uid, id):
                return False

            self.write(cr, uid, [id], {'state' : 'confirmed'})
        return True

    def workflow_action_approve(self, cr, uid, ids, context={}):
        for id in ids:
            self.write(cr, uid, [id], {'state' : 'approved'})
        return True

    def workflow_action_done(self, cr, uid, ids, context={}):
        for id in ids:
            self.write(cr, uid, [id], {'state' : 'done'})
        return True

    def workflow_action_cancel(self, cr, uid, ids, context={}):
        for id in ids:
            self.write(cr, uid, [id], {'state' : 'cancelled'})
        return True

    def button_action_cancel(self, cr, uid, ids, context={}):
        wkf_service = netsvc.LocalService('workflow')
        
        for id in ids:
            if not self.delete_workflow_instance(cr, uid, id):
                return False
                
            if not self.create_workflow_instance(cr, uid, id):
                return False
                
            wkf_service.trg_validate(uid, 'account.cash.statement', id, 'button_cancel', cr)

        return True

    def button_action_set_to_draft(self, cr, uid, ids, context={}):
        for id in ids:
            if not self.delete_workflow_instance(cr, uid, id):
                return False
                
            if not self.create_workflow_instance(cr, uid, id):
                return False

            self.write(cr, uid, [id], {'state' : 'draft'})
            
        return True  

    def create_sequence(self, cr, uid, id):
        """
        """
        obj_sequence = self.pool.get('ir.sequence')
        
        cash_opname = self.browse(cr, uid, [id])[0]

        if cash_opname.name != '/' : return True

        if cash_opname.company_id.sequence_cash_opname_id:
            sequence_id = cash_opname.company_id.sequence_cash_opname_id.id
        else:
            raise osv.except_osv(_('Warning!'),_('Please set sequence for cash opname'))
            return False

        sequence = obj_sequence.next_by_id(cr, uid, sequence_id)
        
        self.write(cr, uid, [id], {'name' : sequence})
        
        return True  

    def check_balance(self, cr, uid, id):
        cash_opname = self.browse(cr, uid, [id])[0]

        if cash_opname.balance_start_compute != cash_opname.balance_start and cash_opname.balance_start != cash_opname.cash_balance_start:
            raise osv.except_osv(_('Warning!'),_('Real starting balance different from computed starting balance')) 
            return False

        if cash_opname.balance_end != cash_opname.balance_end_real and cash_opname.balance_end_rel != cash_opname.cash_balance_end:
            raise osv.except_osv(_('Warning!'),_('Real ending balance different from computed ending balance'))
            return False

        return True

    def delete_workflow_instance(self, cr, uid, id):
        wkf_service = netsvc.LocalService('workflow')
        wkf_service.trg_delete(uid, 'account.cash.statement', id, cr)    
        
        return True
        
    def create_workflow_instance(self, cr, uid, id):
        wkf_service = netsvc.LocalService('workflow')
        wkf_service.trg_create(uid, 'account.cash.statement', id, cr)
        return True


    def onchange_journal_id(self, cr, uid, statement_id, journal_id, date_start, date_end, context=None):
        value = {}
        warning = {}
        domain = {}

        obj_journal = self.pool.get('account.journal')
        obj_move = self.pool.get('account.move.line')
        domain['move_line_ids'] = [('id','=',0)]

        if not journal_id:
            return {'value' : value, 'domain' : domain, 'warning' : warning}


        journal_data = obj_journal.read(cr, uid, journal_id, ['company_id', 'currency', 'default_credit_account_id', 'default_debit_account_id'], context=context)
        company_id = journal_data['company_id']
        currency_id = journal_data['currency'] or self.pool.get('res.company').browse(cr, uid, company_id[0], context=context).currency_id.id
        debit_id = journal_data['default_debit_account_id'][0]
        credit_id = journal_data['default_credit_account_id'][0]

        move_line_ids = []

        if journal_id and date_start and date_end:
            move_line_ids += obj_move.search(cr, uid, [('statement_id','=',False),('account_id','=',debit_id),('date','>=',date_start),('date','<=',date_end)])
            move_line_ids += obj_move.search(cr, uid, [('statement_id','=',False),('account_id','=',credit_id),('date','>=',date_start),('date','<=',date_end)])

        if move_line_ids:
            domain['move_line_ids'] = [('id','in',move_line_ids)]


        value = {
                'company_id' : company_id,
                'currency_id' : currency_id,
                }

        return {'value': value, 'domain' : domain, 'warning' : warning}

cash_statement()




