# -*- encoding: utf-8 -*-
##############################################################################
#    
#    Copyright (C) 2011 OpenERP Italian Community (<http://www.openerp-italia.org>). 
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from report import report_sxw

class Parser(report_sxw.rml_parse):

    def __init__(self, cr, uid, name, context):
        super(Parser, self).__init__(cr, uid, name, context)
        self.data_list = []
        self.journal_list = []
        self.localcontext.update({
                                    'get_journals' : self.get_journals,
                                    'get_data' : self.get_data,
                                    'get_opening_balance' : self.get_opening_balance,
                                    })
    def get_journals(self):
        obj_journal = self.pool.get('account.journal')

        if self.localcontext['data']['form']['journal_ids']:
            #TODO:
            for journal in obj_journal.browse(self.cr, self.uid, self.localcontext['data']['form']['journal_ids']):
                self.journal_list.append({'id' : journal.id, 'name' : journal.name, 'currency' : journal.currency and journal.currency.name or journal.company_id.currency_id.name})
        else:
            journal_ids = self._get_all_journal_ids()

            if journal_ids:
                for journal in obj_journal.browse(self.cr, self.uid, journal_ids):
                    self.journal_list.append({'id' : journal.id, 'name' : journal.name, 'currency' : journal.currency and journal.currency.name or journal.company_id.currency_id.name})

        return self.journal_list

    def _get_all_journal_ids(self):
        obj_journal = self.pool.get('account.journal')

        journal_ids = []

        criteria = [('company_id','=',self.localcontext['data']['form']['company_id'][0]),('type','=','cash')]
        journal_ids = obj_journal.search(self.cr, self.uid, criteria)

        return journal_ids

    def get_opening_balance(self, journal_id):
        obj_account = self.pool.get('account.account')
        obj_journal = self.pool.get('account.journal')
        obj_fiscalyear = self.pool.get('account.fiscalyear')

        journal = obj_journal.browse(self.cr, self.uid, [journal_id])[0]
        fiscalyear = obj_fiscalyear.browse(self.cr, self.uid, [self.localcontext['data']['form']['fiscalyear_id'][0]])[0]
        account_id = journal.default_debit_account_id.id

        ctx =   {
                'date_from' : self.localcontext['data']['form']['date_start'],
                'date_to' : self.localcontext['data']['form']['date_stop'],
                'initial_bal' : True,
                'fiscalyear' : self.localcontext['data']['form']['fiscalyear_id'][0],
                }

        if self.localcontext['data']['form']['state'] != 'all':
            ctx.update({'state' : self.localcontext['data']['form']['state']})

        account = obj_account.browse(self.cr, self.uid, [account_id], context=ctx)[0]

        return journal.currency and account.foreign_balance or account.balance

    def get_data(self, journal_id):
        obj_line = self.pool.get('account.query_cash_in_out')
        obj_journal = self.pool.get('account.journal')

        criteria = [('company_id','=',self.localcontext['data']['form']['company_id'][0])]

        if self.localcontext['data']['form']['date_start']:
            criteria += [('date','>=', self.localcontext['data']['form']['date_start'])]

        if self.localcontext['data']['form']['date_stop']:
            criteria += [('date','<=',self.localcontext['data']['form']['date_stop'])] 

        if self.localcontext['data']['form']['state'] != 'all':
            criteria += [('move_id.state','=',self.localcontext['data']['form']['state'])]


        criteria += [('journal_id','=',journal_id)]

        line_ids = obj_line.search(self.cr, self.uid, criteria)
        self.data_list = []

        journal = obj_journal.browse(self.cr, self.uid, [journal_id])[0]

        if line_ids:
            no = 1
            balance = self.get_opening_balance(journal_id)
            for line in obj_line.browse(self.cr, self.uid, line_ids):
                debit = journal.currency and line.debit_amount_currency or line.debit
                credit = journal.currency and line.credit_amount_currency or line.credit
                balance += (debit - credit)
                res =   {
                        'no' : no,
                        'transaction' : line.move_id.name,
                        'date' : line.date,
                        'description' : line.name,
                        'debit' : debit,
                        'credit' : credit,
                        'balance' : balance,
                        }
                self.data_list.append(res)
                no += 1

        return self.data_list





            

