# -*- encoding: utf-8 -*-
##############################################################################
#    
#    Copyright (C) 2011 OpenERP Italian Community (<http://www.openerp-italia.org>). 
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from report import report_sxw
from datetime import datetime

class Parser(report_sxw.rml_parse):

    def __init__(self, cr, uid, name, context):
        super(Parser, self).__init__(cr, uid, name, context)
        self.journal_list = []
        self.total_amount = 0.0


        self.localcontext.update({
                                    'get_company_name' : self.get_company,
                                    'get_date' : self.get_date,
                                    'get_direction' : self.get_direction,
                                    'get_period_lenght' : self.get_period_lenght,
                                    'get_journals' : self.get_journals,
                                    })

    def get_company(self):
        obj_company = self.pool.get('res.company')
        company_id = self.localcontext['data']['form']['company_id'][0]

        company = obj_company.read(self.cr, self.uid, [company_id], ['name'])[0]

        return company['name']

    def get_journals(self):
        obj_journal = self.pool.get('account.journal')

        if self.localcontext['data']['form']['journal_ids']:
            for journal in obj_journal.browse(self.cr, self.uid, self.localcontext['data']['form']['journal_ids']):
                data = self.get_data(journal.id)

                period_lenght = self.localcontext['data']['form']['period_lenght']
                # total_lenght = period_lenght * 5
                aging_label = {}

                for interval in range(1,6):
                    aging_label['aging%s' % interval] = '%s - %s' % (str(period_lenght * (interval-1)), str(period_lenght * (interval)))

                    if interval == 5:
                        aging_label['aging5'] = '+%s' % (str(period_lenght * (4)))

                result =    {
                            'id' : journal.id,
                            'name' : journal.name,
                            'data' : data['data'],
                            'aging1' : data['aging1'],
                            'aging2' : data['aging2'],
                            'aging3' : data['aging3'],
                            'aging4' : data['aging4'],
                            'aging5' : data['aging5'],
                            'aging_label1' : aging_label['aging1'],
                            'aging_label2' : aging_label['aging2'],
                            'aging_label3' : aging_label['aging3'],
                            'aging_label4' : aging_label['aging4'],
                            'aging_label5' : aging_label['aging5'],
                            }
                
                self.journal_list.append(result)

        return self.journal_list

    def get_date(self):
        date_as_of = self.localcontext['data']['form']['date']
        return 'As of %s' % (datetime.strptime(date_as_of, '%Y-%m-%d').strftime('%d-%m-%Y'))

    def get_direction(self):
        direction = self.localcontext['data']['form']['direction']
        direction_label = direction == 'future' and 'Future Analysis' or 'Past Analysis'
        return direction_label

    def get_period_lenght(self):
        period_lenght = self.localcontext['data']['form']['period_lenght']
        return '%s day(s) period' % (str(period_lenght))

    def get_total_amount(self):
        return self.total_amount

    def get_data(self, journal_id):
        obj_line = self.pool.get('account.query_payable_aging')

        date_as_of = self.localcontext['data']['form']['date']
        direction = self.localcontext['data']['form']['direction']
        fiscalyear_id = self.localcontext['data']['form']['fiscalyear_id'][0]
        period_lenght = self.localcontext['data']['form']['period_lenght']

        criteria = [('move_id.period_id.fiscalyear_id','=',fiscalyear_id),('journal_id.id','=',journal_id),('date',direction == 'future' and '>=' or '<=', date_as_of)]
        context = {'date' : date_as_of, 'period_lenght' : period_lenght}

        data_list = []
        result = {}
        aging1 = aging2 = aging3 = aging4 = aging5 = 0.0

        line_ids = obj_line.search(self.cr, self.uid, criteria)

        if line_ids:
            no = 1
            for line in obj_line.browse(self.cr, self.uid, line_ids, context=context):
                res =   {
                        'no' : no,
                        'date' : datetime.strptime(line.date, '%Y-%m-%d').strftime('%d-%m-%Y'),
                        'date_due' : datetime.strptime(line.date_due, '%Y-%m-%d').strftime('%d-%m-%Y'),
                        'transaction' : line.move_id.name,
                        'description' : line.name,
                        'partner' : line.partner_id.name,
                        'aging1' : line.aging1,
                        'aging2' : line.aging2,
                        'aging3' : line.aging3,
                        'aging4' : line.aging4,
                        'aging5' : line.aging5,
                        }

                data_list.append(res)
                no += 1
                aging1 += line.aging1
                aging2 += line.aging2
                aging3 += line.aging3
                aging4 += line.aging4
                aging5 += line.aging5

        result =    {
                    'data' : data_list,
                    'aging1' : aging1,
                    'aging2' : aging2,
                    'aging3' : aging3,
                    'aging4' : aging4,
                    'aging5' : aging5,
                    }


        return result





            

