# -*- encoding: utf-8 -*-
##############################################################################
#    
#    Copyright (C) 2011 OpenERP Italian Community (<http://www.openerp-italia.org>). 
#    All Rights Reserved
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from report import report_sxw
from datetime import datetime, timedelta

class Parser(report_sxw.rml_parse):

    def __init__(self, cr, uid, name, context):
        super(Parser, self).__init__(cr, uid, name, context)
        self.data_list = []
        self.journal_list = []
        self.total_amount = 0.0
        self.localcontext.update({
                                    'get_data' : self.get_data,
                                    'get_total_amount' : self.get_total_amount,
                                    'get_company' : self.get_company,
                                    'get_date_start' : self.get_date_start,
                                    'get_date_end' : self.get_date_end,
                                    })

    def get_company(self):
        obj_company = self.pool.get('res.company')
        company_id = self.localcontext['data']['form']['company_id'][0]

        company = obj_company.read(self.cr, self.uid, [company_id], ['name'])[0]

        return company['name']

    def get_total_amount(self):
        return self.total_amount

    def get_date_start(self):
        date_start = self.localcontext['data']['form']['date_start']

        return datetime.strptime(date_start, '%Y-%m-%d').strftime('%d-%m-%Y')

    def get_date_end(self):
        date_end = self.localcontext['data']['form']['date_stop']

        return datetime.strptime(date_end, '%Y-%m-%d').strftime('%d-%m-%Y')

    def get_data(self):
        obj_line = self.pool.get('account.query_sales_detail')

        criteria = []

        if self.localcontext['data']['form']['date_start']:
            criteria += [('date','>=', self.localcontext['data']['form']['date_start'])]

        if self.localcontext['data']['form']['date_stop']:
            criteria += [('date','<=',self.localcontext['data']['form']['date_stop'])] 

        if self.localcontext['data']['form']['state'] != 'all':
            criteria += [('state','in',['proforma','posted'])]

        journal_ids = self.localcontext['data']['form']['journal_ids']
        

        criteria += [('journal_id','in',journal_ids)]

        line_ids = obj_line.search(self.cr, self.uid, criteria)
        self.data_list = []

        if line_ids:
            no = 1
            for line in obj_line.browse(self.cr, self.uid, line_ids):
                res =   {
                        'no' : no,
                        'transaction' : line.move_id.name,
                        'account' : '%s %s' % (line.account_id.code,line.account_id.name),
                        'date' : datetime.strptime(line.date, '%Y-%m-%d').strftime('%d-%m-%Y'),
                        'description' : line.name,
                        'customer' : line.partner_id and line.partner_id.name or '-',
                        'amount' : line.credit,
                        'amount_currency' : line.currency_id and line.amount_currency or False,
                        'currency' : line.currency_id.name,
                        'journal' : line.journal_id.name,
                        }

                self.data_list.append(res)
                no += 1
                self.total_amount += line.credit

        return self.data_list





            

