# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import tools
from osv import fields,osv


class query_accrued_expense(osv.osv):
    def function_amount_residual(self, cr, uid, ids, fields_name, args, context=None):
        res = {}

        obj_move_line = self.pool.get('account.move.line')
        date_as_of = context.get('date_as_of', False)

        for move_line in obj_move_line.browse(cr, uid, ids, context={'date' : date_as_of, 'date_as_of':date_as_of}):
            res[move_line.id] = move_line.amount_residual

        return res


    _name = 'account.query_accrued_expense'
    _description = 'Query Accrued Expense'
    _auto = False
    _columns = {
                'name' : fields.char(string='Description', size=64),
                'move_id' : fields.many2one(string='# Move', obj='account.move'),
                'account_id' : fields.many2one(string='Account', obj='account.account'),
                'company_id' : fields.many2one(string='Company', obj='res.company'),
                'date' : fields.date(string='Date'),
                'journal_id' : fields.many2one(string='Journal', obj='account.journal'),
                'partner_id' : fields.many2one(string='Partner', obj='res.partner'),
                'period_id' : fields.many2one(string='Period', obj='account.period'),
                'credit' : fields.float(string='Credit'),
                'debit' : fields.float(string='Amount'),
                'amount_residual' : fields.function(string='Amount Residual', fnct=function_amount_residual, type='float', method=True, store=False),
                'state' : fields.selection(string='State', selection=[('draft','Unposted'),('posted','Posted')]),
                            

                            }

    def init(self, cr):
        tools.drop_view_if_exists(cr, 'account_query_accrued_expense')
        strSQL =    """
                    CREATE OR REPLACE VIEW account_query_accrued_expense AS (
                        SELECT
                                A.id AS id,
                                A.name AS name,
                                A.move_id AS move_id,
                                A.account_id AS account_id,
                                B.company_id AS company_id,
                                B.date as date,
                                B.journal_id AS journal_id,
                                A.partner_id AS partner_id,
                                B.period_id AS period_id,
                                A.credit AS credit,
                                A.debit as debit,
                                B.state AS state
                        FROM account_move_line AS A
                        JOIN account_move AS B ON A.move_id = B.id
                        JOIN account_journal C ON B.journal_id = C.id
                        JOIN account_account D ON A.account_id = D.id
                    )
                    """
        
        
        cr.execute(strSQL)



query_accrued_expense()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
