# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import tools
from osv import fields,osv


class query_bank_voucher_line(osv.osv):

    _name = 'account.query_bank_voucher_line'
    _description = 'Bank Voucher Line'
    _auto = False
    _order = 'voucher_date,voucher_number,id'


    def _function_compute(self, cr, uid, ids, field_name, args, context=None):
        if context is None:
            context = {}

        obj_line = self.pool.get('account.voucher.line')

        res = {}

        for line in obj_line.browse(cr, uid, ids):
            res[line.id] =  {
                            'tax' : 0.0,
                            'after_tax' : 0.0,
                            }

            #RECEIPT
            if line.type == 'cr':
                res[line.id]['tax'] = line.amount_tax > 0 and abs(line.amount_tax) or -1.0 * abs(line.amount_tax)
                res[line.id]['after_tax'] = line.amount_after_tax < 0 and -1.0 * abs(line.amount_after_tax) or abs(line.amount_after_tax)
            #PAYMENT
            else:
                res[line.id]['tax'] = line.amount_tax < 0 and abs(line.amount_tax) or -1.0 * abs(line.amount_tax)
                res[line.id]['after_tax'] = line.amount_after_tax > 0 and -1.0 * abs(line.amount_after_tax) or abs(line.amount_after_tax)

        return res

            



    _columns =  {
                'name' : fields.char(string='Description', size=255, required=False, readonly=False),
                'voucher_number' : fields.char(string='# Voucher', size=255, required=False, readonly=False),
                'product_id' : fields.many2one(string='Product', required=False, readonly=False,
                    obj='product.product'),
                'product_uom_id' : fields.many2one(string='UoM', required=False, readonly=False,
                    obj='product.uom'),
                'product_qty' : fields.float(string='Qty.', required=False, readonly=False),
                'product_unit_price' : fields.float(string='Unit Price', required=False, readonly=False),
                'amount' : fields.float(string='Amount', required=False, readonly=False),
                'tax' : fields.function(string='Tax', type='float', fnct=_function_compute,
                    store=False, method=True, multi='all'),
                'after_tax' : fields.function(string='Tax', type='float', fnct=_function_compute,
                    store=False, method=True, multi='all'),
                'amount_in' : fields.float(string='In', required=False, readonly=False),
                'amount_out' : fields.float(string='Out', required=False, readonly=False),
                'account_id' : fields.many2one(string='Account', required=False, readonly=False,
                    obj='account.account'),
                'move_line_id' : fields.many2one(string='Journal Item', required=False, readonly=False,
                    obj='account.move.line'),
                'period_id' : fields.many2one(string='Period', required=False, readonly=False,
                    obj='account.period'),
                'voucher_id' : fields.many2one(string='# Voucher', required=False, readonly=False,
                    obj='account.voucher'),
                'voucher_date' : fields.date(string='Date', required=False, readonly=False),
                'journal_id' : fields.many2one(string='Journal', required=False, readonly=False,
                    obj='account.journal'),
                'state' : fields.selection(string='State', required=False, readonly=False,
                    selection=[('draft','Draft'),('confirm','Waiting For Approval'),('approve','Ready To Process'),
                        ('proforma','Proforma'),('posted','Posted'),('cancel','Cancelled')]),
                }

    def init(self, cr):
        tools.drop_view_if_exists(cr, 'account_query_bank_voucher_line')
        strSQL =    """
                    CREATE OR REPLACE VIEW account_query_bank_voucher_line AS (
                        SELECT  a.id AS id,
                                a.id AS line_id,
                                a.product_id AS product_id,
                                a.product_uom_id AS product_uom_id,
                                a.product_qty AS product_qty,
                                a.product_unit_price AS product_unit_price,
                                CASE 
                                    WHEN a.type = 'cr' THEN a.amount
                                    ELSE -1.0 * a.amount
                                END AS amount,
                                CASE 
                                    WHEN a.type = 'cr' THEN a.amount
                                    ELSE 0.0
                                END AS amount_in,
                                CASE 
                                    WHEN a.type = 'dr' THEN a.amount
                                    ELSE 0.0
                                END AS amount_out,
                                a.account_id AS account_id,
                                a.voucher_id AS voucher_id,
                                b.period_id AS period_id,
                                a.move_line_id AS move_line_id,
                                a.name AS name,
                                b.date AS voucher_date,
                                b.number AS voucher_number,
                                b.journal_id AS journal_id,
                                b.state AS state
                        FROM    account_voucher_line AS a
                        INNER JOIN  account_voucher AS b ON a.voucher_id = b.id
                        INNER JOIN  (
                                    SELECT  res_id,
                                            module,
                                            model
                                    FROM    ir_model_data
                                    WHERE   module = 'ar_account' AND
                                            model = 'account.voucher_type' AND
                                            (
                                            name = 'data_voucherType_bankPayment' OR
                                            name = 'data_voucherType_bankReceive'
                                            )
                                    ) AS c ON b.voucher_type_id = c.res_id
                        WHERE   b.state != 'cancel' OR
                                b.state != 'draft' OR
                                b.state != 'confirm' OR
                                b.state != 'approve'

                    )
                    """
        
        
        cr.execute(strSQL)






query_bank_voucher_line()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
