# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from tools.translate import _
from openerp.osv import osv, fields
from datetime import datetime

class wizard_query_accrued_income(osv.osv_memory):
    _name = 'account.wizard_query_accrued_income'
    _description = 'Wizard Query Accured Income'
   
    def default_company_id(self, cr, uid, context={}):
        obj_user = self.pool.get('res.users')

        user = obj_user.browse(cr, uid, [uid])[0]

        return user.company_id.id

    def default_state(self, cr, uid, context={}):
        return 'posted'

    def default_date_as_of(self, cr, uid, context={}):
        return datetime.now().strftime('%Y-%m-%d')

    _columns =  {
                'company_id' : fields.many2one(string='Company', obj='res.company', required=True),
                'date_start' : fields.date(string='Start Date', required=False),
                'date_stop' : fields.date(string='End Date', required=False),
                'date_as_of' : fields.date(string='As Of', required=True),
                'state' : fields.selection(string='State', selection=[('all','All'),('draft','Draft'),('posted','Posted')], required=True),
                }

    _defaults = {
                'company_id' : default_company_id,
                'state' : default_state,
                'date_as_of' : default_date_as_of,
                }

    def button_query_report(self, cr, uid, ids, data, context=None):
        obj_data = self.pool.get('ir.model.data')

        account_type_id = []

        if context is None:
            context = {}
 
        wizard = self.browse(cr, uid, ids)[0]

        account_type_id.append(obj_data.get_object_reference(cr, uid, 'l10n_id_account_type', 'accountType_pendapatanDiterimaDimuka')[1])
        account_type_id.append(obj_data.get_object_reference(cr, uid, 'l10n_id_account_type', 'accountType_depositPelanggan')[1])
        account_type_id.append(obj_data.get_object_reference(cr, uid, 'l10n_id_account_type', 'accountType_sewaDiterimaDimuka')[1])

        if not account_type_id:
            raise osv.except_osv(_('Warning!'),_('No account type defined for accrued income'))
            return False

        criteria = [('account_id.user_type.id','in',account_type_id),('credit','>',0),('debit','=',0)]

        if wizard.date_start:
            criteria += [('date','>=',wizard.date_start)]

        if wizard.date_stop:
            criteria += [('date','<=',wizard.date_stop)]

        if wizard.state != 'all':
            criteria += [('state','=',wizard.state)]


        view_id = obj_data.get_object_reference(cr, uid, 'koi_account_query', 'tree_account_queryAccruedIncome')[1]
            
        
        return  {
                'type': 'ir.actions.act_window',
                'name' : 'Payable To Be Settle',
                'res_model' : 'account.query_accrued_income',
                'view_type' : 'form',
                'view_mode' : 'tree',
                'view_id' : view_id,
                'target' : 'current',
                'domain' : criteria,
                'context' : {'date_as_of' : wizard.date_as_of},
                }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
