# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from tools.translate import _
from openerp.osv import osv, fields
from datetime import datetime

class wizard_query_aged_receivable(osv.osv_memory):
    _name = 'account.wizard_query_aged_receivable'
    _description = 'Wizard Aged Receivable'
   
    def default_company_id(self, cr, uid, context={}):
        obj_user = self.pool.get('res.users')

        user = obj_user.browse(cr, uid, [uid])[0]

        return user.company_id.id


    def default_date_as_of(self, cr, uid, context={}):
        return datetime.now().strftime('%Y-%m-%d')

    def default_data_type(self, cr, uid, context={}):
        return 'summary'

    def default_currency_type(self, cr, uid, context={}):
        return 'base'

    def default_period_lenght(self, cr, uid, context={}):
        return 30

    _columns =  {
                'company_id' : fields.many2one(string='Company', obj='res.company', required=True),
                'partner_ids' : fields.many2many(string='Customers', obj='res.partner'),
                'data_type' : fields.selection(string='Detail Level', selection=[('summary','Summary'),('detail','Detail')], required=True),
                'currency_type' : fields.selection(string='Currency', selection=[('base','Base Currency'),('respesctive','Respective Currency')], required=True),
                'date_as_of' : fields.date(string='As Of', required=True),
                'period_lenght' : fields.integer(string='Period Lenght (days)', required=True),
                }

    _defaults = {
                'company_id' : default_company_id,
                'date_as_of' : default_date_as_of,
                'data_type' : default_data_type,
                'currency_type' : default_currency_type,
                'period_lenght' : default_period_lenght,
                }

    def button_query(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('Warning!'), _('On-screen query still not available'))

    def button_print(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('Warning'), _('Printed report still not available'))

    def button_query_report(self, cr, uid, ids, data, context=None):
        obj_data = self.pool.get('ir.model.data')

        account_type_id = []

        if context is None:
            context = {}
 
        wizard = self.browse(cr, uid, ids)[0]

        account_type_id.append(obj_data.get_object_reference(cr, uid, 'l10n_id_account_type', 'accountType_biayaDibayarDimuka')[1])

        if not account_type_id:
            raise osv.except_osv(_('Warning!'),_('No account type defined for accrued expense'))
            return False

        criteria = [('account_id.user_type.id','in',account_type_id),('credit','<',0),('debit','>',0)]

        if wizard.date_start:
            criteria += [('date','>=',wizard.date_start)]

        if wizard.date_stop:
            criteria += [('date','<=',wizard.date_stop)]

        if wizard.state != 'all':
            criteria += [('state','=',wizard.state)]


        view_id = obj_data.get_object_reference(cr, uid, 'koi_account_query', 'tree_account_queryAccruedExpense')[1]
            
        
        return  {
                'type': 'ir.actions.act_window',
                'name' : 'Accrued Expense',
                'res_model' : 'account.query_accrued_expense',
                'view_type' : 'form',
                'view_mode' : 'tree',
                'view_id' : view_id,
                'target' : 'current',
                'domain' : criteria,
                'context' : {'date' : wizard.date_as_of},
                }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
