# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from openerp.osv import osv, fields
from openerp.tools.translate import _

class wizard_query_payable_aging(osv.osv_memory):
    _name = 'account.wizard_query_payable_aging'
    _description = 'Wizard Query Payable Aging'
   
    def default_company_id(self, cr, uid, context={}):
        obj_user = self.pool.get('res.users')

        user = obj_user.browse(cr, uid, [uid])[0]

        return user.company_id.id

    def default_direction(self, cr, uid, context={}):
        return 'future'

    def default_period_lenght(self, cr, uid, context={}):
        return 30

    _columns =  {
                'company_id' : fields.many2one(string='Company', obj='res.company', required=True),
                'fiscalyear_id' : fields.many2one(string='Fiscal Year', obj='account.fiscalyear',
                    required=True),
                'date' : fields.date(string='Start Date', required=True),
                'direction' : fields.selection(string='Direction', selection=[('past','Past'),('future','Future')], required=True),
                'period_lenght' : fields.integer(string='Period Length (days)', required=True),
                'journal_ids' : fields.many2many(string='Journals', obj='account.journal',
                    rel='rel_wzd_payable_aging_2_journal', id1='wizard_id', id2='journal_id'),
                }

    _defaults = {
                'company_id' : default_company_id,
                'direction' : default_direction,
                'period_lenght' : default_period_lenght,
                }

    def button_query_report(self, cr, uid, ids, data, context=None):
        obj_data = self.pool.get('ir.model.data')
        obj_line = self.pool.get('account.move.line')
        valid_ids = []

        if context is None:
            context = {}

        wizard = self.browse(cr, uid, ids)[0]
        dict_wizard = self.read(cr, uid, ids, ['journal_ids'])[0]

        criteria = [('move_id.period_id.fiscalyear_id','=',wizard.fiscalyear_id.id),('journal_id.id','in',dict_wizard['journal_ids']),('date',wizard.direction == 'future' and '>=' or '<=', wizard.date)]
        line_ids = obj_line.search(cr, uid, criteria)


        if line_ids:
            for line in obj_line.browse(cr, uid, line_ids, context={'date' : wizard.date}):
                if line.amount_residual != 0.0:
                    valid_ids.append(line.id)


        view_id = obj_data.get_object_reference(cr, uid, 'koi_account_query', 'tree_account_queryPayableAging')[1]
            
        
        return  {
                'type': 'ir.actions.act_window',
                'name' : 'Payable Aging',
                'res_model' : 'account.query_payable_aging',
                'view_type' : 'form',
                'view_mode' : 'tree',
                'view_id' : view_id,
                'target' : 'current',
                'context' : {'period_lenght' : wizard.period_lenght, 'date' : wizard.date},
                'domain' : [('id','in',valid_ids)],
                }

    def button_print_ods(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('Warning!'),_('This feature is not implemented yet'))
        return False

    def button_print_xls(self, cr, uid, ids, context=None):
        raise osv.except_osv(_('Warning!'),_('This feature is not implemented yet'))
        return False
# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
